import enum
from typing import List, Union

from typing_extensions import Literal

from slark.types._common import BaseModel


class FieldType(enum.Enum):
    """https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-field/guide
    可选值有：

    1：多行文本
    2：数字
    3：单选
    4：多选
    5：日期
    7：复选框
    11：人员
    13：电话号码
    15：超链接
    17：附件
    18：单向关联
    20：公式
    21：双向关联
    22：地理位置
    23：群组
    1001：创建时间
    1002：最后更新时间
    1003：创建人
    1004：修改人
    1005：自动编号"""

    TEXT = 1
    NUMBER = 2
    SINGLE_SELECT = 3
    MULTI_SELECT = 4
    DATE = 5
    CHECKBOX = 7
    PERSON = 11
    PHONE = 13
    URL = 15
    ATTACHMENT = 17
    LOOKUP = 18
    FORMULA = 20
    DUPLEX_LINK = 21
    LOCATION = 22
    GROUP_CHAT = 23
    CREATED_TIME = 1001
    MODIFIED_TIME = 1002
    CREATED_USER = 1003
    MODIFIED_USER = 1004
    AUTO_NUMBER = 1005


class UIType(enum.Enum):
    """字段在界面上的展示类型，例如进度字段是数字的一种展示形态

    示例值："Progress"

    可选值有：

    Text：多行文本
    Barcode：条码
    Number：数字
    Progress：进度
    Currency：货币
    Rating：评分
    SingleSelect：单选
    MultiSelect：多选
    DateTime：日期
    Checkbox：复选框
    User：人员
    GroupChat：群组
    Phone：电话号码
    Url：超链接
    Attachment：附件
    SingleLink：单向关联
    Formula：公式
    DuplexLink：双向关联
    Location：地理位置
    CreatedTime：创建时间
    ModifiedTime：最后更新时间
    CreatedUser：创建人
    ModifiedUser：修改人
    AutoNumber：自动编号"""

    TEXT = "Text"
    BARCODE = "Barcode"
    NUMBER = "Number"
    PROGRESS = "Progress"
    CURRENCY = "Currency"
    RATING = "Rating"
    SINGLE_SELECT = "SingleSelect"
    MULTI_SELECT = "MultiSelect"
    DATE_TIME = "DateTime"
    CHECKBOX = "Checkbox"
    USER = "User"
    GROUP_CHAT = "GroupChat"
    PHONE = "Phone"
    URL = "Url"
    ATTACHMENT = "Attachment"
    SINGLE_LINK = "SingleLink"
    FORMULA = "Formula"
    DUPLEX_LINK = "DuplexLink"
    LOCATION = "Location"
    CREATED_TIME = "CreatedTime"
    MODIFIED_TIME = "ModifiedTime"
    CREATED_USER = "CreatedUser"
    MODIFIED_USER = "ModifiedUser"
    AUTO_NUMBER = "AutoNumber"


class FieldPropertyEditModes(BaseModel):
    manual: bool = True
    """是否允许手动录入"""
    scan: bool = True
    """是否允许移动端录入"""


class BarCodeFieldProperty(BaseModel):
    allowed_edit_modes: FieldPropertyEditModes


class NumberFieldProperty(BaseModel):
    formatter: str
    """格式，参考：formatter
    网页中数字格式	formatter
    整数	"0"
    保留1位小数	"0.0"
    保留2位小数	"0.00"
    保留3位小数	"0.000"
    保留4位小数	"0.0000"
    千分位	"1,000"
    千分位（小数点）	"1,000.00"
    百分比	"%"
    百分比（小数点）	"0.00%"
    人民币	"¥"
    人民币（小数点）	"¥0.00"
    美元	"$"
    美元（小数点）	"$0.00"
    """


class CurrencyFieldProperty(BaseModel):
    formatter: Literal["0", "0.0", "0.00", "0.000", "0.0000"]
    """"0"
    "0.0"
    "0.00"
    "0.000"
    "0.0000"
    """

    currency_code: Literal[
        "CNY",
        "USD",
        "EUR",
        "GBP",
        "AED",
        "AUD",
        "BRL",
        "CAD",
        "CHF",
        "HKD",
        "INR",
        "IDR",
        "JPY",
        "KRW",
        "MOP",
        "MXN",
        "MYR",
        "PHP",
        "PLN",
        "RUB",
        "SGD",
        "THB",
        "TRY",
        "TWD",
        "VND",
    ]
    """网页中数字格式	货币符号	货币说明
    "CNY"	¥	人民币
    "USD"	$	美元
    "EUR"	€	欧元
    "GBP"	£	英镑
    "AED"	dh	阿联酋迪拉姆
    "AUD"	$	澳大利亚元
    "BRL"	R$	巴西雷亚尔
    "CAD"	$	加拿大元
    "CHF"	CHF	瑞士法郎
    "HKD"	$	港元
    "INR"	₹	印度卢比
    "IDR"	Rp	印尼盾
    "JPY"	¥	日元
    "KRW"	"₩"	韩元
    "MOP"	MOP$	澳门元
    "MXN"	$	墨西哥比索
    "MYR"	RM	马来西亚令吉
    "PHP"	₱	菲律宾比索
    "PLN"	zł	波兰兹罗提
    "RUB"	₽	俄罗斯卢布
    "SGD"	$	新加坡元
    "THB"	฿	泰国铢
    "TRY"	₺	土耳其里拉
    "TWD"	NT$	新台币
    "VND"	₫	越南盾"""


class ProgressFieldProperty(BaseModel):
    formatter: str
    """进度的格式
    可选值：
    "0"
    "0.0"
    "0.00"
    "0%"
    "0.0%"
    "0.00%"
    """

    range_customize: bool = False
    """是否允许自定义进度条值"""
    min: Union[int, float, None] = None
    """进度的最小值, range_customizeture为true时必填"""
    max: Union[int, float, None] = None
    """进度的最大值, range_customize为true时必填"""


class RatingFieldPropertyRating(BaseModel):
    symbol: Literal[
        "star", "heart", "thumbsup", "fire", "smile", "lightning", "flower", "number"
    ] = "star"
    """评分图标
    "star"
    "heart"
    "thumbsup"
    "fire"
    "smile"
    "lightning"
    "flower"
    "number"
    """


class RatingFieldProperty(BaseModel):
    formatter: str = "0"
    rating: Union[RatingFieldPropertyRating, None] = None
    """评分设置"""
    min: Union[int, None] = None
    """评分的最小值"""
    max: Union[int, None] = None


class OptionFieldPropertyOption(BaseModel):
    id: Union[str, None] = None
    """选项id"""
    name: Union[str, None] = None
    """选项名"""
    color: Union[int, None] = None
    """选项颜色, 0~54, 从上一个选项的color开始依次递增"""


class OptionFieldProperty(BaseModel):
    options: List[OptionFieldPropertyOption]


class DateTimeFieldProperty(BaseModel):
    date_formatter: Literal[
        "yyyy/MM/dd",
        "yyyy/MM/dd HH:mm",
        "yyyy-MM-dd",
        "yyyy-MM-dd HH:mm",
        "MM-dd",
        "MM/dd/yyyy",
        "dd/MM/yyyy",
    ] = "yyyy/MM/dd"
    """日期格式，参考：日期格式date_formatter
    网页中日期格式	date_formatter
    2021/01/30	"yyyy/MM/dd"
    2021/01/30 14:00	"yyyy/MM/dd HH:mm"
    2021-01-30	"yyyy-MM-dd"
    2021-01-30 14:00	"yyyy-MM-dd HH:mm"
    01-30	"MM-dd"
    01/30/2021	"MM/dd/yyyy"
    30/01/2021	"dd/MM/yyyy"
    """
    auto_fill: bool = False
    """新纪录自动填写创建时间"""


class PersonFieldProperty(BaseModel):
    multiple: bool = True
    """允许添加多个成员"""


class LookupFieldProperty(BaseModel):
    multiple: bool = True
    """允许添加多个记录"""
    table_id: str
    """关联的数据表的id"""
    table_name: Union[str, None] = None
    """关联的数据表的名字（仅在响应体中返回）"""


class DuplexLinkFieldProperty(BaseModel):
    back_field_id: Union[str, None] = None
    """关联的数据表中双向关联字段的id（仅在响应体中返回）"""
    back_field_name: Union[str, None] = None
    """关联的数据表中双向关联字段的名字, default to: 关联的数据表名-本字段名"""
    multiple: bool = True
    """允许添加多个记录"""
    table_id: str
    """关联的数据表的id"""
    table_name: Union[str, None] = None
    """关联的数据表的名字（仅在响应体中返回）"""


class FormulaFieldProperty(BaseModel):
    formatter: Literal[
        "",
        "0",
        "0.0",
        "0.00",
        "1,000",
        "1,000.00",
        "%",
        "0.00%",
        "¥",
        "¥0.00",
        "$",
        "$0.00",
        "yyyy/MM/dd HH:mm",
        "yyyy/MM/dd",
        "yyyy-MM-dd",
        "MM-dd",
    ] = ""
    """格式参考：formatter
    网页中公式格式	formatter
    默认	""
    整数	"0"
    保留1位小数	"0.0"
    保留2位小数	"0.00"
    千分位	"1,000"
    千分位（小数点）	"1,000.00"
    百分比	"%"
    百分比（小数点）	"0.00%"
    人民币	"¥"
    人民币（小数点）	"¥0.00"
    美元	"$"
    美元（小数点）	"$0.00"
    2021/01/30 14:00	"yyyy/MM/dd HH:mm"
    2021/01/30	"yyyy/MM/dd"
    2021-01-30	"yyyy-MM-dd"
    01-30	"MM-dd"
    """

    formula_expression: str
    """格式参考：[formula](https://www.feishu.cn/hc/zh-CN/articles/360049067853-%E5%A4%9A%E7%BB%B4%E8%A1%A8%E6%A0%BC%E5%85%AC%E5%BC%8F%E5%AD%97%E6%AE%B5%E6%A6%82%E8%BF%B0)"""


class GroupFieldProperty(BaseModel):
    multiple: bool = True
    """是否允许添加多个群组"""


class CreateUpdateTimeFieldProperty(BaseModel):
    date_formatter: Literal[
        "yyyy/MM/dd",
        "yyyy/MM/dd HH:mm",
        "yyyy-MM-dd",
        "yyyy-MM-dd HH:mm",
        "MM-dd",
        "MM/dd/yyyy",
        "dd/MM/yyyy",
    ] = "yyyy/MM/dd"
    """日期格式，参考：日期格式date_formatter"""


class AutoNumberFieldPropertyAutoSerial(BaseModel):
    type: Literal["custom", "auto_increment_number"]
    """网页中编号类型	auto_serial_type
    自定义编号	"custom"
    自增数字	"auto_increment_number"
    """


class AutoNumberFieldPropertyOption(BaseModel):
    type: Literal["system_number", "fixed_text", "created_time"]
    """网页中编号类型	rule_option_type	对应的value限制
    自增数字	"system_number"	1-9
    固定字符	"fixed_text"	最大长度：20
    创建日期	"created_time"	可选值参考：created_time
    
    网页中创建日期	created_time
    20220130	"yyyyMMdd"
    202201	"yyyyMM"
    2022	"yyyy"
    0130	"MMdd"
    01	"MM"
    30	"dd"
    """
    value: str


class AutoNumberFieldProperty(BaseModel):
    auto_serial: Union[AutoNumberFieldPropertyAutoSerial, None] = None
    reformat_existing_records: bool = False
    options: List[AutoNumberFieldPropertyOption]


FieldPropertyType = Union[
    BarCodeFieldProperty,
    NumberFieldProperty,
    CurrencyFieldProperty,
    ProgressFieldProperty,
    RatingFieldProperty,
    OptionFieldProperty,
    DateTimeFieldProperty,
    PersonFieldProperty,
    LookupFieldProperty,
    DuplexLinkFieldProperty,
    FormulaFieldProperty,
    GroupFieldProperty,
    CreateUpdateTimeFieldProperty,
    AutoNumberFieldProperty,
]



import enum

from .base import BaseModel


class PlainText(BaseModel):
    tag: str = "plain_text"
    content: str
    """文本的内容，最多支持 100 个字符。"""


class TextSize(enum.Enum):
    HEADING_0 = "heading-0"
    """特大标题（30px）"""
    HEADING_1 = "heading-1"
    """一级标题（24px）"""
    HEADING_2 = "heading-2"
    """二级标题（20 px）"""
    HEADING_3 = "heading-3"
    """三级标题（18px）"""
    HEADING_4 = "heading-4"
    """四级标题（16px）"""
    HEADING = "heading"
    """标题（16px）"""
    NORMAL = "normal"
    """正文（14px）"""
    NOTATION = "notation"
    """辅助信息（12px）"""
    XXXX_LARGE = "xxxx-large"
    """30px"""
    XXX_LARGE = "xxx-large"
    """24px"""
    XX_LARGE = "xx-large"
    """20px"""
    X_LARGE = "x-large"
    """18px"""
    LARGE = "large"
    """16px"""
    MEDIUM = "medium"
    """14px"""
    SMALL = "small"
    """12px"""
    X_SMALL = "x-small"
    """10px"""


import json
from typing import List, Union

from typing_extensions import Literal

from slark.types._common import BaseModel


class LarkUserID(BaseModel):
    union_id: str
    user_id: str
    open_id: str


class LarkMessageMention(BaseModel):
    key: str
    id: LarkUserID
    name: str
    tenant_key: str


class TextContent(BaseModel):
    text: str


class FileContent(BaseModel):
    file_key: str
    file_name: str


class AudioContent(BaseModel):
    file_key: str
    duration: int


class MediaContent(BaseModel):
    file_key: str
    image_key: str
    file_name: str
    duration: int


class StickerContent(BaseModel):
    file_key: str


class RichTextItem(BaseModel):
    tag: Literal["text", "a", "at", "img", "file", "media", "emotion", "hr", "code_block"]
    text: Union[str, None] = None
    style: Union[List[str], None] = None
    href: Union[str, None] = None
    user_id: Union[str, None] = None
    user_name: Union[str, None] = None
    image_key: Union[str, None] = None
    file_key: Union[str, None] = None
    language: Union[str, None] = None
    un_escape: Union[bool, None] = None
    emoji_type: Union[str, None] = None
    duration: Union[int, None] = None


class RichTextContent(BaseModel):
    title: str
    content: List[List[RichTextItem]]

    def get_raw_text(self):
        raw_text = ""
        for line in self.content:
            for item in line:
                if item.tag == "text":
                    raw_text += item.text
                elif item.tag == "a":
                    raw_text += item.text
                elif item.tag == "at":
                    raw_text += f"@{item.user_name}"
                elif item.tag == "hr":
                    raw_text += "----"
                elif item.tag == "code_block":
                    raw_text += f"```{item.language}\n{item.text}\n```"
            raw_text += "\n"
        return raw_text


class ImageContent(BaseModel):
    image_key: str


LARK_MSG_TYPE = Union[
    Literal["text", "post", "file", "folder", "image", "audio", "media", "sticker"], str
]

LARK_MSG_TYPE_MAP = {
    "text": TextContent,
    "file": FileContent,
    "audio": AudioContent,
    "media": MediaContent,
    "sticker": StickerContent,
    "post": RichTextContent,
    "image": ImageContent,
}


class LarkGetMessageSender(BaseModel):
    id: str
    id_type: str
    sender_type: str
    tenant_key: str


class LarkGetMessageBody(BaseModel):
    content: Union[
        str,
        TextContent,
        RichTextContent,
        FileContent,
        AudioContent,
        MediaContent,
        StickerContent,
        ImageContent,
    ]


class LarkGetMessageMention(BaseModel):
    key: str
    id: str
    id_type: str
    name: str
    tenant_key: str


class LarkGetMessage(BaseModel):
    message_id: str
    root_id: Union[str, None] = None
    parent_id: Union[str, None] = None
    thread_id: Union[str, None] = None
    msg_type: LARK_MSG_TYPE
    create_time: str
    update_time: str
    deleted: bool
    updated: bool
    chat_id: Union[str, None] = None
    sender: LarkGetMessageSender
    body: LarkGetMessageBody
    mentions: Union[List[LarkGetMessageMention], None] = None
    upper_message_id: Union[str, None] = None

    def model_post_init(self, __context) -> None:
        if self.msg_type in LARK_MSG_TYPE_MAP:
            self.body.content = LARK_MSG_TYPE_MAP[self.msg_type].model_validate_json(
                self.body.content
            )
        else:
            self.body.content = json.loads(self.body.content)

    def get_plain_text(self):
        if self.msg_type == "text":
            return self.body.content.text
        elif self.msg_type == "post":
            return self.body.content.get_raw_text()
        else:
            return "Unsupported message type"


class LarkEventMessageBody(BaseModel):
    message_id: str
    """message_id 是一条消息的唯一标识。当发送一条消息时，系统会自动生成唯一ID。message_id 的格式是以om_ 开头的字符串，例如：om_934be5776f5a87239a298af9e74c0f72"""
    root_id: Union[str, None] = None
    parent_id: Union[str, None] = None
    create_time: str
    update_time: str
    chat_id: Union[str, None] = None
    """消息所在的群组 ID"""
    thread_id: Union[str, None] = None
    """消息所属的话题 ID（不返回说明该消息非话题消息），说明参见：话题介绍"""
    chat_type: Literal["p2p", "group"]
    message_type: LARK_MSG_TYPE
    content: Union[
        str,
        TextContent,
        RichTextContent,
        FileContent,
        AudioContent,
        MediaContent,
        StickerContent,
        ImageContent,
    ]
    mentions: Union[List[LarkMessageMention], None] = None
    user_agent: Union[str, None] = None
    """用户代理数据，仅在接收事件的机器人具备获取客户端用户代理信息权限时返回"""

    def model_post_init(self, __context) -> None:
        if self.message_type in LARK_MSG_TYPE_MAP:
            self.content = LARK_MSG_TYPE_MAP[self.message_type].model_validate_json(self.content)
        else:
            self.content = json.loads(self.content)

    def get_plain_text(self):
        if self.message_type == "text":
            return self.content.text
        elif self.message_type == "post":
            return self.content.get_raw_text()
        else:
            return "Unsupported message type"



from typing import Union

import httpx

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.api_path import API_PATH
from slark.types.bitables.field.common import (
    FieldDescription,
    FieldPropertyType,
    FieldType,
    UIType,
)
from slark.types.bitables.field.request import (
    CreateFieldBody,
    CreateFieldParams,
    ListFieldParams,
    UpdateFieldBody,
)
from slark.types.bitables.field.response import (
    CreateFieldResponse,
    DeleteFieldResponse,
    ListFieldResponse,
    UpdateFieldResponse,
)


class AsyncField(AsyncAPIResource):
    async def list(
        self,
        app_token: str,
        *,
        table_id: str,
        view_id: Union[str, None] = None,
        text_field_as_array: Union[bool, None] = None,
        page_token: Union[str, None] = None,
        page_size: Union[int, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> ListFieldResponse:
        """根据 app_token 和 table_id，获取数据表的所有字段

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-field/list
        Args:
            app_token (str): 多维表格的 app_token
            table_id (str): 多维表格的 table_id
            view_id (Union[str, None], optional): 视图 ID. 示例值："vewOVMEXPF". Defaults to None.
            text_field_as_array (Union[bool, None], optional): 	控制字段描述（多行文本格式）数据的返回格式, true 表示以数组富文本形式返回。\
                示例值：true. Defaults to None.
            page_token (Union[str, None], optional): 分页标记，第一次请求不填，表示从头开始遍历；\
                分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果。示例值："fldwJ4YrtB". Defaults to None.
            page_size (Union[int, None], optional): 分页大小，最大支持 100。示例值：10. Defaults to 20. 
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            ListFieldResponse: 数据表的所有字段
        """
        return await self._get(
            API_PATH.bitables.list_field.format(app_token=app_token, table_id=table_id),
            options={
                "timeout": timeout,
                "params": ListFieldParams(
                    view_id=view_id,
                    text_field_as_array=text_field_as_array,
                    page_token=page_token,
                    page_size=page_size,
                ).model_dump(),
            },
            cast_to=ListFieldResponse,
        )

    async def create(
        self,
        app_token: str,
        *,
        table_id: str,
        field_name: str,
        type: FieldType,
        property: Union[FieldPropertyType, None] = None,
        ui_type: Union[UIType, None] = None,
        disable_sync: Union[bool, None] = None,
        text: Union[FieldDescription, None] = None,
        client_token: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> CreateFieldResponse:
        """该接口用于在数据表中新增一个字段
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-field/create

        Args:
            app_token (str): 多维表格的 app_token
            table_id (str): 多维表格的 table_id
            field_name (str): 多维表格字段名。请注意：名称中的首尾空格将会被去除。示例值："多行文本"
            type (FieldType): 多维表格字段类型
            property (Union[FieldPropertyType, None], optional): 字段属性. Defaults to None.
            ui_type (Union[UIType, None], optional): 字段在界面上的展示类型，例如进度字段是数字的一种展示形态. Defaults to None.
            disable_sync (Union[bool, None], optional): 是否禁止同步，如果为true，\
                表示禁止同步该描述内容到表单的问题描述（只在新增、修改字段时生效）. Defaults to None.
            text (Union[FieldDescription, None], optional): 字段的描述. Defaults to None.
            client_token (Union[str, None], optional): 格式为标准的 uuidv4，操作的唯一标识，用于幂等的进行更新操作。\
                此值为空表示将发起一次新的请求，此值非空表示幂等的进行更新操作。\
                示例值："fe599b60-450f-46ff-b2ef-9f6675625b97"
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Raises:
            ValueError: disable_sync and text must be set together

        Returns:
            CreateFieldResponse: 新增字段的返回值
        """
        if disable_sync is not None and text is not None:
            desp = FieldDescription(disable_sync=disable_sync, text=text)
        elif disable_sync is None and text is not None or disable_sync is not None and text is None:
            raise ValueError("disable_sync and text must be set together")
        else:
            desp = None
        return await self._post(
            API_PATH.bitables.create_field.format(app_token=app_token, table_id=table_id),
            body=CreateFieldBody(
                field_name=field_name,
                type=type,
                property=property,
                ui_type=ui_type,
                description=desp,
            ).model_dump(),
            options={
                "timeout": timeout,
                "params": CreateFieldParams(client_token=client_token).model_dump(),
            },
            cast_to=CreateFieldResponse,
        )

    async def update(
        self,
        app_token: str,
        *,
        table_id: str,
        field_id: str,
        field_name: str,
        type: FieldType,
        property: Union[FieldPropertyType, None] = None,
        ui_type: Union[UIType, None] = None,
        disable_sync: Union[bool, None] = None,
        text: Union[FieldDescription, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> UpdateFieldResponse:
        """该接口用于在数据表中更新一个字段
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-field/update

        Args:
            app_token (str): 多维表格的 app_token
            table_id (str): 多维表格的 table_id
            field_id (str): 多维表格的 field_id
            field_name (str): 多维表格字段名，名称中的首尾空格将会被去除。示例值："多行文本"。
            type (FieldType): 多维表格字段类型
            property (Union[FieldPropertyType, None], optional): 字段属性. Defaults to None.
            ui_type (Union[UIType, None], optional): 字段在界面上的展示类型，例如进度字段是数字的一种展示形态. Defaults to None.
            disable_sync (Union[bool, None], optional): 是否禁止同步，如果为true，\
                表示禁止同步该描述内容到表单的问题描述（只在新增、修改字段时生效）. Defaults to None.
            text (Union[FieldDescription, None], optional): 字段描述内容. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Raises:
            ValueError: disable_sync and text must be set together

        Returns:
            UpdateFieldResponse: 更新字段的返回值
        """
        if disable_sync is not None and text is not None:
            desp = FieldDescription(disable_sync=disable_sync, text=text)
        elif disable_sync is None and text is not None or disable_sync is not None and text is None:
            raise ValueError("disable_sync and text must be set together")
        else:
            desp = None
        return await self._put(
            API_PATH.bitables.update_field.format(
                app_token=app_token,
                table_id=table_id,
                field_id=field_id,
            ),
            body=UpdateFieldBody(
                field_name=field_name,
                type=type,
                property=property,
                ui_type=ui_type,
                description=desp,
            ).model_dump(),
            options={"timeout": timeout},
            cast_to=UpdateFieldResponse,
        )

    async def delete(
        self,
        app_token: str,
        *,
        table_id: str,
        field_id: str,
    ) -> DeleteFieldResponse:
        """该接口用于在数据表中删除一个字段
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-field/delete

        Args:
            app_token (str): 多维表格的 app_token
            table_id (str): 多维表格的 table_id
            field_id (str): 多维表格的 field_id

        Returns:
            DeleteFieldResponse: 删除字段的返回值
        """
        return await self._delete(
            API_PATH.bitables.delete_field.format(
                app_token=app_token, table_id=table_id, field_id=field_id
            ),
            cast_to=DeleteFieldResponse,
        )


class Field(APIResource):
    def list(
        self,
        app_token: str,
        *,
        table_id: str,
        view_id: Union[str, None] = None,
        text_field_as_array: Union[bool, None] = None,
        page_token: Union[str, None] = None,
        page_size: Union[int, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> ListFieldResponse:
        """根据 app_token 和 table_id，获取数据表的所有字段

        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-field/list
        Args:
            app_token (str): 多维表格的 app_token
            table_id (str): 多维表格的 table_id
            view_id (Union[str, None], optional): 视图 ID. 示例值："vewOVMEXPF". Defaults to None.
            text_field_as_array (Union[bool, None], optional): 	控制字段描述（多行文本格式）数据的返回格式, true 表示以数组富文本形式返回。\
                示例值：true. Defaults to None.
            page_token (Union[str, None], optional): 分页标记，第一次请求不填，表示从头开始遍历；\
                分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果。示例值："fldwJ4YrtB". Defaults to None.
            page_size (Union[int, None], optional): 分页大小，最大支持 100。示例值：10. Defaults to 20. 
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            ListFieldResponse: 数据表的所有字段
        """
        return self._get(
            API_PATH.bitables.list_field.format(app_token=app_token, table_id=table_id),
            options={
                "timeout": timeout,
                "params": ListFieldParams(
                    view_id=view_id,
                    text_field_as_array=text_field_as_array,
                    page_token=page_token,
                    page_size=page_size,
                ).model_dump(),
            },
            cast_to=ListFieldResponse,
        )

    def create(
        self,
        app_token: str,
        *,
        table_id: str,
        field_name: str,
        type: FieldType,
        property: Union[FieldPropertyType, None] = None,
        ui_type: Union[UIType, None] = None,
        disable_sync: Union[bool, None] = None,
        text: Union[FieldDescription, None] = None,
        client_token: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> CreateFieldResponse:
        """该接口用于在数据表中新增一个字段
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-field/create

        Args:
            app_token (str): 多维表格的 app_token
            table_id (str): 多维表格的 table_id
            field_name (str): 多维表格字段名。请注意：名称中的首尾空格将会被去除。示例值："多行文本"
            type (FieldType): 多维表格字段类型
            property (Union[FieldPropertyType, None], optional): 字段属性. Defaults to None.
            ui_type (Union[UIType, None], optional): 字段在界面上的展示类型，例如进度字段是数字的一种展示形态. Defaults to None.
            disable_sync (Union[bool, None], optional): 是否禁止同步，如果为true，\
                表示禁止同步该描述内容到表单的问题描述（只在新增、修改字段时生效）. Defaults to None.
            text (Union[FieldDescription, None], optional): 字段的描述. Defaults to None.
            client_token (Union[str, None], optional): 格式为标准的 uuidv4，操作的唯一标识，用于幂等的进行更新操作。\
                此值为空表示将发起一次新的请求，此值非空表示幂等的进行更新操作。\
                示例值："fe599b60-450f-46ff-b2ef-9f6675625b97"
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Raises:
            ValueError: disable_sync and text must be set together

        Returns:
            CreateFieldResponse: 新增字段的返回值
        """
        if disable_sync is not None and text is not None:
            desp = FieldDescription(disable_sync=disable_sync, text=text)
        elif disable_sync is None and text is not None or disable_sync is not None and text is None:
            raise ValueError("disable_sync and text must be set together")
        else:
            desp = None
        return self._post(
            API_PATH.bitables.create_field.format(app_token=app_token, table_id=table_id),
            body=CreateFieldBody(
                field_name=field_name,
                type=type,
                property=property,
                ui_type=ui_type,
                description=desp,
            ).model_dump(),
            options={
                "timeout": timeout,
                "params": CreateFieldParams(client_token=client_token).model_dump(),
            },
            cast_to=CreateFieldResponse,
        )

    def update(
        self,
        app_token: str,
        *,
        table_id: str,
        field_id: str,
        field_name: str,
        type: FieldType,
        property: Union[FieldPropertyType, None] = None,
        ui_type: Union[UIType, None] = None,
        disable_sync: Union[bool, None] = None,
        text: Union[FieldDescription, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> UpdateFieldResponse:
        """该接口用于在数据表中更新一个字段
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-field/update

        Args:
            app_token (str): 多维表格的 app_token
            table_id (str): 多维表格的 table_id
            field_id (str): 多维表格的 field_id
            field_name (str): 多维表格字段名，名称中的首尾空格将会被去除。示例值："多行文本"。
            type (FieldType): 多维表格字段类型
            property (Union[FieldPropertyType, None], optional): 字段属性. Defaults to None.
            ui_type (Union[UIType, None], optional): 字段在界面上的展示类型，例如进度字段是数字的一种展示形态. Defaults to None.
            disable_sync (Union[bool, None], optional): 是否禁止同步，如果为true，\
                表示禁止同步该描述内容到表单的问题描述（只在新增、修改字段时生效）. Defaults to None.
            text (Union[FieldDescription, None], optional): 字段描述内容. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Raises:
            ValueError: disable_sync and text must be set together

        Returns:
            UpdateFieldResponse: 更新字段的返回值
        """
        if disable_sync is not None and text is not None:
            desp = FieldDescription(disable_sync=disable_sync, text=text)
        elif disable_sync is None and text is not None or disable_sync is not None and text is None:
            raise ValueError("disable_sync and text must be set together")
        else:
            desp = None
        return self._put(
            API_PATH.bitables.update_field.format(
                app_token=app_token,
                table_id=table_id,
                field_id=field_id,
            ),
            body=UpdateFieldBody(
                field_name=field_name,
                type=type,
                property=property,
                ui_type=ui_type,
                description=desp,
            ).model_dump(),
            options={"timeout": timeout},
            cast_to=UpdateFieldResponse,
        )

    def delete(
        self,
        app_token: str,
        *,
        table_id: str,
        field_id: str,
    ) -> DeleteFieldResponse:
        """该接口用于在数据表中删除一个字段
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table-field/delete

        Args:
            app_token (str): 多维表格的 app_token
            table_id (str): 多维表格的 table_id
            field_id (str): 多维表格的 field_id

        Returns:
            DeleteFieldResponse: 删除字段的返回值
        """
        return self._delete(
            API_PATH.bitables.delete_field.format(
                app_token=app_token, table_id=table_id, field_id=field_id
            ),
            cast_to=DeleteFieldResponse,
        )



from typing import Dict, List, Union

import arrow
import pandas as pd

from slark.types.bitables.field.common import Field, UIType
from slark.types.bitables.record.common import Empty
from slark.types.bitables.record.response import FieldValueType, RecordResponseData


def fields_records_to_dataframe(
    fields: List[Field],
    records: List[RecordResponseData],
    timezone: Union[str, None] = "Asia/Shanghai",
) -> pd.DataFrame:
    def field_value_to_text(field: FieldValueType) -> str:
        if hasattr(field, "text"):
            return field.text
        if isinstance(field, list):
            return ", ".join([field_value_to_text(f) for f in field])
        if isinstance(field, Empty):
            return None
        return field

    data = [{k: field_value_to_text(v) for k, v in record.fields.items()} for record in records]
    index = [record.record_id for record in records]
    df = pd.DataFrame(data, index=index)
    for field in fields:
        if field.ui_type in [
            UIType.DATE_TIME.value,
            UIType.CREATED_TIME.value,
            UIType.MODIFIED_TIME.value,
        ]:
            df[field.field_name] = (
                pd.to_datetime(df[field.field_name], unit="ms") + arrow.now(timezone).utcoffset()
            )

    return df


def dataframe_to_records(
    df: pd.DataFrame,
    timezone: Union[str, None] = "Asia/Shanghai",
    use_index_as_record_id: bool = False,
) -> List[Dict[str, FieldValueType]]:
    for col in df.columns:
        if df[col].dtype == "datetime64[ns]":
            # 转为 unix timestamp，单位为 ms
            df[col] = df[col].dt.tz_localize(timezone).astype(int) // 10**6
    df_records = df.to_dict(orient="records")
    if use_index_as_record_id:
        return [
            {"record_id": index, "fields": record} for index, record in zip(df.index, df_records)
        ]
    else:
        return df_records



from typing import List, Union

import slark.types.card as card


def build_card(
    elements: List[card.ElementType], header: Union[card.CardHeader, None] = None
) -> card.InteractiveCard:
    _header = {} if header is None else header
    return card.InteractiveCard(
        elements=elements,
        header=_header,
    )


def build_heading_text_element(content: str):
    return card.PlainTextElement(
        content=content,
        text_size=card.TextSize.HEADING,
        text_align="left",
    )


def build_single_column_set_element(elements: List[card.ElementType]) -> card.ColumnSetElement:
    return card.ColumnSetElement(columns=[card.ColumnElement(elements=elements)])


def build_header_with_icon(title: str, subtitle: str, template: str, icon: str) -> card.CardHeader:
    return card.CardHeader(
        title=card.CardHeaderTitle(content=title),
        subtitle=card.CardHeaderSubtitle(content=subtitle),
        template=template,
        ud_icon=card.UDIconElement(token=icon),
    )


def build_success_msg_card(msg: str, title: str, subtitle: str):
    return build_card(
        elements=[
            build_single_column_set_element(
                [card.DivElement(text=build_heading_text_element("Message"))]
            ),
            card.MDElement(content=f"```txt\n{msg}\n```"),
        ],
        header=build_header_with_icon(
            title, subtitle, card.CardTemplate.GREEN, card.UDIconToken.YES_FILLED
        ),
    )


def build_error_msg_card(
    msg: str, traceback: str, title: str, subtitle: str
) -> card.InteractiveCard:
    return build_card(
        elements=[
            build_single_column_set_element(
                [card.DivElement(text=build_heading_text_element("Message"))]
            ),
            card.MDElement(content=f"```txt\n{msg}\n```"),
            build_single_column_set_element(
                [card.DivElement(text=build_heading_text_element("Traceback"))]
            ),
            card.MDElement(content=f"```\n{traceback.strip()}\n```"),
        ],
        header=build_header_with_icon(
            title, subtitle, card.CardTemplate.RED, card.UDIconToken.ERROR_FILLED
        ),
    )



from .assets.assets import Assets, AsyncAssets
from .auth.auth import AsyncAuth, Auth
from .bitable.bitable import AsyncBiTable, BiTable
from .board.board import AsyncBoard, Board
from .documents.documents import AsyncDocuments, Documents
from .drive.drive import AsyncDrive, Drive
from .knowledge_space.knowledge_space import AsyncKnowledgeSpace, KnowledgeSpace
from .messages.messages import AsyncMessages, Messages
from .sheets.sheets import AsyncSpreadsheets, Spreadsheets
from .webhook.webhook import AsyncWebhook, Webhook

__all__ = [
    "Auth",
    "Webhook",
    "KnowledgeSpace",
    "Spreadsheets",
    "BiTable",
    "Documents",
    "Board",
    "Assets",
    "Messages",
    "Drive",
    "AsyncAuth",
    "AsyncWebhook",
    "AsyncKnowledgeSpace",
    "AsyncSpreadsheets",
    "AsyncBiTable",
    "AsyncDocuments",
    "AsyncBoard",
    "AsyncAssets",
    "AsyncMessages",
    "AsyncDrive",
]



from typing import List, Literal, Union

from typing_extensions import TYPE_CHECKING

from ..base import BaseElement
from ..behavior import BehaviorType
from ..color import Color
from ..confirm import ConfirmDialogue
from ..displaying.image import ImageElement
from ..displaying.markdown import MDElement
from ..displaying.note import NoteElement
from ..displaying.plain_text import DivElement
from ..interactive.checker import CheckerElement
from ..text import PlainText

if TYPE_CHECKING:
    from .column import ColumnSetElement


InteractiveContainerElementType = Union[
    DivElement,
    MDElement,
    ImageElement,
    NoteElement,
    "ColumnSetElement",
    CheckerElement,
    "InteractiveContainerElement",
]


class InteractiveContainerElement(BaseElement):
    tag: str = "interactive_container"
    width: Union[str, Literal["full", "auto"], None] = None
    """交互容器的宽度。可取值：

    fill：卡片最大支持宽度
    auto：自适应宽度
    [16,999]px：自定义宽度，如 "20px"。最小宽度为 16px"""

    height: Union[str, Literal["auto"], None] = None
    """交互容器的高度。可取值：

    auto：自适应高度
    [10,999]px：自定义高度，如 "20px"""

    background_style: Union[Literal["default", "laser"], Color, None] = None
    """交互容器的背景色样式。可取值：

    default：默认的白底样式，客户端深色主题下为黑底
    laser：镭射渐变彩色样式
    卡片支持的颜色枚举值和 RGBA 语法自定义颜色。参考颜色枚举值"""

    has_border: Union[bool, None] = None
    """是否展示边框，粗细固定为 1px。"""

    border_color: Union[Color, None] = None
    """边框的颜色，仅 has_border 为 true 时，此字段生效。枚举值为卡片支持的颜色枚举值和 RGBA 语法自定义颜色，参考颜色枚举值。"""

    corner_radius: Union[str, None] = None
    """交互容器的圆角半径，单位是像素（px）或百分比（%）。取值遵循以下格式：

    [0,∞]px，如 "10px"
    [0,100]%，如 "30%" """

    padding: Union[str, None] = None
    """交互容器的内边距。值的取值范围为 [0,28]px。支持填写单值或多值：

    单值：如 "10px"，表示容器内四个内边距都为 10px
    多值：如 "4px 12px 4px 12px"，表示容器内上、右、下、左的内边距分别为 4px，12px，4px，12px。四个值必填，使用空格间隔"""

    behaviors: BehaviorType
    """https://open.feishu.cn/document/uAjLw4CM/ukzMukzMukzM/feishu-cards/configuring-card-interactions#355ec8c0"""

    hover_tips: Union[PlainText, None] = None
    """用户在 PC 端将光标悬浮在交互容器上方时的文案提醒。默认为空。"""

    disabled: Union[bool, None] = None
    """是否禁用交互容器。可选值：

    true：禁用整个容器
    false：容器组件保持可用状态"""

    disabled_tips: Union[PlainText, None] = None
    """禁用交互容器后，用户触发交互时的弹窗文案提醒。默认为空，即不弹窗。"""

    confirm: Union[ConfirmDialogue, None] = None
    """二次确认弹窗配置。指在用户提交时弹出二次确认弹窗提示；只有用户点击确认后，才提交输入的内容。该字段默认提供了确认和取消按钮，你只需要配置弹窗的标题与内容即可。"""

    elements: List[InteractiveContainerElementType]



from typing import Dict, Union

from typing_extensions import Literal

from ..._common import BaseModel
from ..base import BaseElement
from ..confirm import ConfirmDialogue
from ..text import PlainText


class InputBoxFallback(BaseModel):
    tag: str = "fallback_text"
    text: Union[PlainText, None] = None


class InputBoxElement(BaseElement):
    tag: str = "input"
    name: Union[str, None] = None
    """输入框的唯一标识。当输入框内嵌在表单容器时，该属性生效，用于识别用户提交的文本属于哪个输入框。\n
    注意：当输入框组件嵌套在表单容器中时，该字段必填且需在卡片全局内唯一。"""
    required: Union[bool, None] = None
    """输入框的内容是否必填。当输入框内嵌在表单容器时，该属性可用。其它情况将报错或不生效。可取值：\n
    true：输入框必填。当用户点击表单容器的“提交”时，未填写输入框，则前端提示“有必填项未填写”，不会向开发者的服务端发起回传请求。\n
    false：输入框选填。当用户点击表单容器的“提交”时，未填写输入框，仍提交表单容器中的数据。"""
    disabled: Union[bool, None] = None
    """是否禁用该输入框。该属性仅支持飞书 V7.4 及以上版本的客户端。可选值：\n
    true：禁用输入框组件\n
    false：输入框组件保持可用状态"""
    placeholder: Union[PlainText, None] = None
    """输入框中的占位文本。占位文本的内容，最多支持 100 个字符。"""
    default_value: Union[str, None] = None
    """输入框中为用户预填写的内容。展示为用户在输入框中输入文本后待提交的样式。"""
    width: Union[str, None] = None
    """输入框的宽度。支持以下枚举值：\n
    default：默认宽度\n
    fill：卡片最大支持宽度\n
    [100,∞)px：自定义宽度。超出卡片宽度时将按最大支持宽度展示"""
    max_length: Union[int, None] = None
    """输入框可容纳的最大文本长度，可取 1~1,000 范围内的整数。当用户输入的文本字符数超过最大文本长度，组件将报错提示。"""
    input_type: Union[Literal["text", "multiline_text", "password"], None] = None
    """指定输入框的输入类型。默认为 text，即文本类型。支持以下枚举值：
    text：普通文本\n
    multiline_text：多行文本，即可输入包含换行符的多行文本内容。换行符在回调中以 \\n 返回\n
    password：密码。用户输入的文本内容将以“•”显示"""
    show_icon: Union[bool, None] = None
    """当输入类型为密码类型时，是否展示如下所示的前缀图标。仅当 input_type 为 password 时有效。"""
    rows: Union[int, None] = None
    """当输入类型为多行文本时，输入框的默认展示行数。仅当 input_type 为 multiline_text 时有效。"""
    auto_resize: Union[bool, None] = None
    """当输入类型为多行文本时，输入框高度是否自适应文本高度。仅在 PC 端生效。仅当 input_type 为 multiline_text 时有效。可选值：\n
    true：输入框高度自适应输入框内的文本高度\n
    false：输入框高度固定为 rows 属性指定的高度，不随输入框中的内容变化而变化。"""
    max_rows: Union[int, None] = None
    """输入框的最大展示行数。仅当 auto_resize 为 true 时有效。注意：\n
    取值为大于等于 1 的整数。否则，小于 1 则自动取 1，不为整数则四舍五入取整。
    取值为空时不限制输入框的最大文本展示高度（默认值），但前端渲染时，输入框可展示的最大高度不超过 x 行。"""
    label: Union[PlainText, None] = None
    """文本标签，即对输入框的描述，用于提示用户要填写的内容。多用于表单容器中内嵌的输入框组件。"""
    label_position: Union[Literal["top", "left"], None] = None
    """文本标签的位置。可取值：\n
    top：文本标签位于输入框上方\n
    left：文本标签位于输入框左边\n
    注意：在移动端等窄屏幕场景下，文本标签将自适应固定展示在输入框上方。"""
    value: Union[str, Dict, None] = None
    """你可在交互事件中自定义回传数据，支持 string 或 object 数据类型。"""
    confirm: Union[ConfirmDialogue, None] = None
    """二次确认弹窗配置。指在用户提交时弹出二次确认弹窗提示；只有用户点击确认后，才提交输入的内容。该字段默认提供了确认和取消按钮，你只需要配置弹窗的标题与内容即可。\n
    注意：confirm 字段仅在用户点击包含提交属性的按钮时才会触发二次确认弹窗。"""
    fallback: Union[InputBoxFallback, Literal["drop"], None] = None
    """设置输入框组件的降级文案。由于输入框仅支持飞书 V6.8 及以上版本的客户端，你需选择在低于此版本的客户端上，该组件的降级展示方式：\n
    不填写该字段，使用系统默认的降级文案：“请升级至最新版本客户端，以查看内容”\n
    "drop"：填写 "drop"，在旧版本客户端上直接丢弃该输入框组件\n
    使用 text 文本对象自定义降级文案"""



import re
from typing import List, Union

import httpx
from pydantic import BaseModel
from typing_extensions import Literal

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.api_path import API_PATH
from slark.types._utils import cached_property
from slark.types.documents.blocks.common import BlockInfo
from slark.types.documents.request import GetRawContentParams
from slark.types.documents.response import GetRawContentResponse

from .blocks import AsyncBlocks, Blocks
from .helper import PageHelper


class DocumentInfo(BaseModel):
    document_id: str


class AsyncDocuments(AsyncAPIResource):
    @cached_property
    def blocks(self) -> AsyncBlocks:
        return AsyncBlocks(self._client)

    async def get_raw_content(
        self,
        document_id: str,
        *,
        lang: Union[Literal[0, 1, 2], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetRawContentResponse:
        """获取文档的纯文本内容。
        应用频率限制：单个应用调用频率上限为每秒 5 次，超过该频率限制，\
            接口将返回 HTTP 状态码 400 及错误码 99991400。\
            当请求被限频，应用需要处理限频状态码，并使用指数退避算法或其它一些频控策略降低对 API 的调用速率。

        Args:
            document_id (str): 文档的唯一标识。
            lang (Union[Literal[0, 1, 2], None], optional): 指定返回的 MentionUser 即 @用户 的语言。可选值有：\n
                0：该用户的默认名称。如：@张敏\n
                1：该用户的英文名称。如：@Min Zhang\n
                2：暂不支持该枚举，使用时返回该用户的默认名称\n
                默认值：0. Defaults to None.
                
        Returns:
            GetRawContentResponse: 返回的文档内容。
        """
        return await self._get(
            API_PATH.documents.get_raw_content.format(document_id=document_id),
            cast_to=GetRawContentResponse,
            options={
                "timeout": timeout,
                "params": GetRawContentParams(lang=lang).model_dump(),
            },
        )

    async def get_document_info(self, url: str) -> DocumentInfo:
        """从文档分享链接中提取 document_id

        Args:
            url (str): URL of the document

        Returns:
            DocumentInfo: document_id
        """
        if "/docx/" in url:
            document_id = re.findall(r"\/docx\/([^\/\?]+)", url)[0]
        elif "/wiki/" in url:
            node_token = re.findall(r"\/wiki\/([^\/\?]+)", url)[0]
            node_info = await self._client.knowledge_space.nodes.get_node_info(node_token)
            if node_info.data.node.obj_type != "docx":
                raise ValueError(f"The node {node_info.data.node.obj_type} is not a document")
            document_id = node_info.data.node.obj_token
        else:
            raise ValueError("The url is not a document url")
        assert document_id, "Document token is not found"

        return DocumentInfo(document_id=document_id)

    async def read_raw(
        self,
        url: str,
        *,
        lang: Union[Literal[0, 1, 2], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> str:
        """从文档中读取纯文本内容

        Args:
            url (str): 文档分享链接
            lang (Union[Literal[0, 1, 2], None], optional): 指定返回的 MentionUser 即 @用户 的语言。可选值有：\n
                0：该用户的默认名称。如：@张敏\n
                1：该用户的英文名称。如：@Min Zhang\n
                2：暂不支持该枚举，使用时返回该用户的默认名称\n
                默认值：0. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): 超时时间. Defaults to None.

        Returns:
            str: 文档内容
        """
        document_id = (await self.get_document_info(url)).document_id
        return (await self.get_raw_content(document_id, lang=lang, timeout=timeout)).data.content

    async def _read_all_blocks(
        self,
        url: str,
        *,
        document_revision_id: Union[int, None] = None,
        user_id_type: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> List[BlockInfo]:
        document_id = (await self.get_document_info(url)).document_id
        page_token = None
        has_more = True
        blocks: List[BlockInfo] = []
        while has_more:
            response = await self.blocks.get_all(
                document_id,
                page_token=page_token,
                document_revision_id=document_revision_id,
                user_id_type=user_id_type,
                timeout=timeout,
            )
            blocks.extend(response.data.items)
            has_more = response.data.has_more
            page_token = response.data.page_token
        return blocks

    async def read_markdown(
        self,
        url: str,
        *,
        assets_path: str = "./assets/",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> str:
        """从文档中读取 markdown 内容

        Args:
            url (str): 文档分享链接。
            assets_path (str, optional): 保存图片的目录. Defaults to "./assets/".
            timeout (Union[httpx.Timeout, None], optional): 超时时间. Defaults to None.

        Returns:
            str: markdown 内容
        """
        blocks = await self._read_all_blocks(url, timeout=timeout)
        return await PageHelper(blocks, client=self._client).to_markdown(assets_path=assets_path)


class Documents(APIResource):
    @cached_property
    def blocks(self) -> Blocks:
        return Blocks(self._client)

    def get_raw_content(
        self,
        document_id: str,
        *,
        lang: Union[Literal[0, 1, 2], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetRawContentResponse:
        """获取文档的纯文本内容。
        应用频率限制：单个应用调用频率上限为每秒 5 次，超过该频率限制，\
            接口将返回 HTTP 状态码 400 及错误码 99991400。\
            当请求被限频，应用需要处理限频状态码，并使用指数退避算法或其它一些频控策略降低对 API 的调用速率。

        Args:
            document_id (str): 文档的唯一标识。
            lang (Union[Literal[0, 1, 2], None], optional): 指定返回的 MentionUser 即 @用户 的语言。可选值有：\n
                0：该用户的默认名称。如：@张敏\n
                1：该用户的英文名称。如：@Min Zhang\n
                2：暂不支持该枚举，使用时返回该用户的默认名称\n
                默认值：0. Defaults to None.
                
        Returns:
            GetRawContentResponse: 返回的文档内容。
        """
        return self._get(
            API_PATH.documents.get_raw_content.format(document_id=document_id),
            cast_to=GetRawContentResponse,
            options={
                "timeout": timeout,
                "params": GetRawContentParams(lang=lang).model_dump(),
            },
        )

    def get_document_info(self, url: str) -> DocumentInfo:
        """从文档分享链接中提取 document_id

        Args:
            url (str): URL of the document

        Returns:
            DocumentInfo: document_id
        """
        if "/docx/" in url:
            document_id = re.findall(r"\/docx\/([^\/\?]+)", url)[0]
        elif "/wiki/" in url:
            node_token = re.findall(r"\/wiki\/([^\/\?]+)", url)[0]
            node_info = self._client.knowledge_space.nodes.get_node_info(node_token)
            if node_info.data.node.obj_type != "docx":
                raise ValueError(f"The node {node_info.data.node.obj_type} is not a document")
            document_id = node_info.data.node.obj_token
        else:
            raise ValueError("The url is not a document url")
        assert document_id, "Document token is not found"

        return DocumentInfo(document_id=document_id)

    def read_raw(
        self,
        url: str,
        *,
        lang: Union[Literal[0, 1, 2], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> str:
        """从文档中读取纯文本内容

        Args:
            url (str): 文档分享链接
            lang (Union[Literal[0, 1, 2], None], optional): 指定返回的 MentionUser 即 @用户 的语言。可选值有：\n
                0：该用户的默认名称。如：@张敏\n
                1：该用户的英文名称。如：@Min Zhang\n
                2：暂不支持该枚举，使用时返回该用户的默认名称\n
                默认值：0. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): 超时时间. Defaults to None.

        Returns:
            str: 文档内容
        """
        document_id = self.get_document_info(url).document_id
        return (self.get_raw_content(document_id, lang=lang, timeout=timeout)).data.content

    def _read_all_blocks(
        self,
        url: str,
        *,
        document_revision_id: Union[int, None] = None,
        user_id_type: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> List[BlockInfo]:
        document_id = (self.get_document_info(url)).document_id
        page_token = None
        has_more = True
        blocks: List[BlockInfo] = []
        while has_more:
            response = self.blocks.get_all(
                document_id,
                page_token=page_token,
                document_revision_id=document_revision_id,
                user_id_type=user_id_type,
                timeout=timeout,
            )
            blocks.extend(response.data.items)
            has_more = response.data.has_more
            page_token = response.data.page_token
        return blocks

    async def read_markdown(
        self,
        url: str,
        *,
        assets_path: str = "./assets/",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> str:
        """从文档中读取 markdown 内容

        Args:
            url (str): 文档分享链接。
            assets_path (str, optional): 保存图片的目录. Defaults to "./assets/".
            timeout (Union[httpx.Timeout, None], optional): 超时时间. Defaults to None.

        Returns:
            str: markdown 内容
        """
        blocks = await self._read_all_blocks(url, timeout=timeout)
        return await PageHelper(blocks, client=self._client).to_markdown(assets_path=assets_path)



from typing import List, Union

from slark.types._common import BaseModel
from slark.types.response import BaseResponse


class CreateTableResponseData(BaseModel):
    table_id: str
    """多维表格数据表的唯一标识符"""
    default_view_id: Union[str, None] = None
    """默认表格视图的id，该字段仅在请求参数中填写了default_view_name或fields才会返回"""
    field_id_list: Union[List[str], None] = None
    """数据表初始字段的id列表，该字段仅在请求参数中填写了fields才会返回"""


class ListTableItem(BaseModel):
    table_id: str
    """多维表格数据表的唯一标识符"""
    revision: int
    """数据表的版本号"""
    name: str
    """数据表的名称"""


class ListTableResponseData(BaseModel):
    has_more: bool
    """是否还有更多项"""
    page_token: Union[str, None] = None
    """分页标记，当 has_more 为 true 时，会同时返回新的 page_token，否则不返回 page_token"""
    total: int
    """总数"""
    items: List[ListTableItem]
    """数据表信息"""


class ListTableResponse(BaseResponse):
    data: ListTableResponseData


class CreateTableResponse(BaseResponse):
    data: CreateTableResponseData


class BatchCreateTableResponseData(BaseModel):
    table_ids: List[str]


class BatchCreateTableResponse(BaseResponse):
    data: BatchCreateTableResponseData


class UpdateTableResponseData(BaseModel):
    name: str
    """数据表的名称"""


class UpdateTableResponse(BaseResponse):
    data: UpdateTableResponseData



from typing import Union

import anyio
import httpx
import pandas as pd
from httpx import Response
from PIL import Image, ImageChops

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.api_path import API_PATH


def trim(im_path: str, save_path: str):
    im = Image.open(im_path)
    bg = Image.new(im.mode, im.size, im.getpixel((0, 0)))
    diff = ImageChops.difference(im, bg)
    diff = ImageChops.add(diff, diff, 2.0, -100)
    bbox = diff.getbbox()
    if bbox:
        im = im.crop(bbox)
        im.save(save_path)


class AsyncBoard(AsyncAPIResource):
    async def download_as_image(
        self,
        whiteboard_id: str,
        *,
        save_dir: str,
        auto_trim: bool = True,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> pd.DataFrame:
        """下载白板为图片。
        通过此接口，可以将白板下载为图片，支持下载为 png、jpg、jpeg 格式。

        Args:
            whiteboard_id (str): 白板 ID。
            save_dir (str): 保存图片的目录。
            auto_trim (bool, optional): 是否自动裁剪图片。 Defaults to True.
            timeout (Union[httpx.Timeout, None], optional): 超时时间。 Defaults to None.

        Returns:
            str: 保存的图片路径。
        """
        response = await self._get(
            API_PATH.board.download_as_image.format(whiteboard_id=whiteboard_id),
            options={
                "timeout": timeout,
                "headers": {"Content-Type": "image/png"},
                "raw_response": True,
            },
            cast_to=Response,
        )

        save_dir_path = anyio.Path(save_dir)
        await save_dir_path.mkdir(exist_ok=True)
        file_path = save_dir_path / f"{whiteboard_id}.png"
        f = await anyio.open_file(file_path, "wb")
        await f.write(response.content)
        await f.aclose()
        if auto_trim:
            trim(file_path, file_path)
        return file_path


class Board(APIResource):
    def download_as_image(
        self,
        whiteboard_id: str,
        *,
        save_dir: str,
        auto_trim: bool = True,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> pd.DataFrame:
        """下载白板为图片。
        通过此接口，可以将白板下载为图片，支持下载为 png、jpg、jpeg 格式。

        Args:
            whiteboard_id (str): 白板 ID。
            save_dir (str): 保存图片的目录。
            auto_trim (bool, optional): 是否自动裁剪图片。 Defaults to True.
            timeout (Union[httpx.Timeout, None], optional): 超时时间。 Defaults to None.

        Returns:
            str: 保存的图片路径。
        """
        response = self._get(
            API_PATH.board.download_as_image.format(whiteboard_id=whiteboard_id),
            options={
                "timeout": timeout,
                "headers": {"Content-Type": "image/png"},
                "raw_response": True,
            },
            cast_to=Response,
        )

        save_dir_path = anyio.Path(save_dir)
        save_dir_path.mkdir(exist_ok=True)
        file_path = save_dir_path / f"{whiteboard_id}.png"
        with file_path.open("wb") as f:
            f.write(response.content)
        if auto_trim:
            trim(file_path, file_path)
        return file_path



from typing import Dict, Union

from typing_extensions import Literal

from .._common import BaseModel
from ..card import I18nText, InteractiveCard


class TemplateCard(BaseModel):
    template_id: str
    template_variable: Union[Dict, None] = None
    template_version_name: Union[str, None] = None

class CallbackCard(BaseModel):
    type: Literal["raw", "template"]
    data: Union[InteractiveCard, TemplateCard]


class CallbackToast(BaseModel):
    type: Literal["info", "success", "error", "warning"]
    """info、success、error、和 warning。"""
    content: Union[str, None] = None
    """提示内容。"""
    i18n: Union[I18nText, None] = None


class CallbackResponse(BaseModel):
    toast: Union[CallbackToast, None] = None
    card: Union[CallbackCard, None] = None



from time import time
from typing import Union

import httpx

from slark.resources.api_path import API_PATH
from slark.types.auth import (
    GetTenantAccessTokenParams,
    GetTenantAccessTokenResponse,
    TenantAccessToken,
)

from .._resources import APIResource, AsyncAPIResource

NETWORK_DELAY_AJUSTMENT = 5


class AsyncToken(AsyncAPIResource):
    async def get_tenant_access_token(
        self, timeout: Union[httpx.Timeout, None] = None
    ) -> TenantAccessToken:
        response = await self._post(
            API_PATH.auth.get_tenant_access_token,
            cast_to=GetTenantAccessTokenResponse,
            body=GetTenantAccessTokenParams.model_validate(
                self._client.app_credentials
            ).model_dump(),
            options={"timeout": timeout, "no_auth": True},
        )
        return TenantAccessToken(
            access_token=response.tenant_access_token,
            expires_at=response.expire + int(time()) - NETWORK_DELAY_AJUSTMENT,
        )


class Token(APIResource):
    def get_tenant_access_token(
        self, timeout: Union[httpx.Timeout, None] = None
    ) -> TenantAccessToken:
        response = self._post(
            API_PATH.auth.get_tenant_access_token,
            cast_to=GetTenantAccessTokenResponse,
            body=GetTenantAccessTokenParams.model_validate(
                self._client.app_credentials
            ).model_dump(),
            options={"timeout": timeout, "no_auth": True},
        )
        return TenantAccessToken(
            access_token=response.tenant_access_token,
            expires_at=response.expire + int(time()) - NETWORK_DELAY_AJUSTMENT,
        )



import time
from typing import TYPE_CHECKING

import anyio

if TYPE_CHECKING:
    from slark.client.lark import AsyncLark
    from slark.client.sync_lark import Lark


class AsyncAPIResource:
    _client: "AsyncLark"

    def __init__(self, client: "AsyncLark"):
        self._client = client
        self._get = client.get
        self._post = client.post
        self._put = client.put
        self._delete = client.delete
        self._patch = client.patch

    async def _sleep(self, seconds: float):
        await anyio.sleep(seconds)


class APIResource:
    _client: "Lark"

    def __init__(self, client: "Lark"):
        self._client = client
        self._get = client.get
        self._post = client.post
        self._put = client.put
        self._delete = client.delete
        self._patch = client.patch

    def _sleep(self, seconds: float):
        time.sleep(seconds)



from slark.types._common import BaseModel
from slark.types.messages.common import LarkEventMessageBody, LarkUserID


class LarkEventSender(BaseModel):
    sender_id: LarkUserID
    sender_type: str
    tenant_key: str
    """tenant key，为租户在飞书上的唯一标识，用来换取对应的tenant_access_token，也可以用作租户在应用里面的唯一标识"""


class MessageEvent(BaseModel):
    sender: LarkEventSender
    message: LarkEventMessageBody


from typing import List, Union

import httpx
from typing_extensions import Literal

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.api_path import API_PATH
from slark.types.bitables.table.common import TableData
from slark.types.bitables.table.request import (
    BatchCreateTableBody,
    BatchCreateTableParams,
    BatchDeleteTableBody,
    CreateTableBody,
    ListTableParams,
    UpdateTableBody,
)
from slark.types.bitables.table.response import (
    BatchCreateTableResponse,
    CreateTableResponse,
    ListTableResponse,
    UpdateTableResponse,
)
from slark.types.response import BaseResponse


class AsyncTable(AsyncAPIResource):
    async def list(
        self,
        app_token: str,
        *,
        timeout: Union[httpx.Timeout, None] = None,
        page_token: Union[str, None] = None,
        page_size: Union[int, None] = None,
    ) -> ListTableResponse:
        """根据 app_token，获取多维表格下的所有数据表
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/list

        Args:
            app_token (str): 多维表格的 app_token
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.
            page_token (Union[str, None], optional): 分页标记，第一次请求不填，表示从头开始遍历；\
                分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果。\
                示例值："tblsRc9GRRXKqhvW". Defaults to None.
            
            page_size (Union[int, None], optional): 分页大小，示例值：10，默认值：20，数据校验规则：\
                最大值：100. Defaults to None.

        Returns:
            ListTableResponse: 数据表列表
        """
        return await self._get(
            API_PATH.bitables.list_tables.format(app_token=app_token),
            options={
                "timeout": timeout,
                "params": ListTableParams(
                    page_token=page_token,
                    page_size=page_size,
                ).model_dump(),
            },
            cast_to=ListTableResponse,
        )

    async def create(
        self,
        app_token: str,
        *,
        table: TableData,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> CreateTableResponse:
        """新增数据表
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/create

        Args:
            app_token (str): 多维表格的 app_token
            table (TableData): 表格数据
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            CreateTableResponse: 创建返回值
        """
        return await self._post(
            API_PATH.bitables.create_table.format(app_token=app_token),
            body=CreateTableBody(
                table=table,
            ).model_dump(),
            options={"timeout": timeout},
            cast_to=CreateTableResponse,
        )

    async def batch_create(
        self,
        app_token: str,
        *,
        names: List[str],
        user_id_type: Union[Literal["user_id", "open_id", "union_id"], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BatchCreateTableResponse:
        """新增多个数据表。
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/batch_create

        Args:
            app_token (str): 多维表格的唯一标识符
            names (List[str]): 数据表名称，注意名称中的首尾空格将会被去除。长度范围：1 字符 ～ 100 字符。
            user_id_type (Union[Literal[&quot;user_id&quot;, &quot;open_id&quot;, &quot;union_id&quot;], None], optional): 用户 ID 类型. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BatchCreateTableResponse: 创建返回值
        """
        return await self._post(
            API_PATH.bitables.batch_create_table.format(app_token=app_token),
            body=BatchCreateTableBody(tables=[{"name": name} for name in names]).model_dump(),
            cast_to=BatchCreateTableResponse,
            options={
                "timeout": timeout,
                "params": BatchCreateTableParams(user_id_type=user_id_type).model_dump(),
            },
        )

    async def update(
        self,
        app_token: str,
        *,
        table_id: str,
        name: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> UpdateTableResponse:
        r"""该接口用于更新数据表的基本信息，包括数据表的名称等。

        Args:
            app_token (str): 多维表格的 app_token
            table_id (str): 多维表格的 table_id
            name (str): 数据表的新名称。请注意：名称中的首尾空格将会被去除。
                如果名称为空或和旧名称相同，接口仍然会返回成功，但是名称不会被更改。\n
                示例值："数据表的新名称"。\
                数据校验规则：长度范围：1 字符 ～ 100 字符。正则校验：^[^\[\]\:\\\/\?\*]+$。
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            UpdateTableResponse: 更新返回值
        """
        return await self._patch(
            API_PATH.bitables.update_table.format(app_token=app_token, table_id=table_id),
            body=UpdateTableBody(name=name).model_dump(),
            cast_to=UpdateTableResponse,
            options={"timeout": timeout},
        )

    async def delete(
        self,
        app_token: str,
        *,
        table_id: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BaseResponse:
        """删除数据表
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/delete

        Args:
            app_token (str): 多维表格的 app_token
            table_id (str): 多维表格的 table_id
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BaseResponse: 删除返回值
        """
        return await self._delete(
            API_PATH.bitables.delete_table.format(app_token=app_token, table_id=table_id),
            options={"timeout": timeout},
            cast_to=BaseResponse,
        )

    async def batch_delete(
        self,
        app_token: str,
        *,
        table_ids: List[str],
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BaseResponse:
        """批量删除数据表
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/batch_delete

        Args:
            app_token (str): 多维表格的 app_token
            table_ids (List[str]): 待删除的数据表的 id
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BaseResponse: 删除返回值
        """
        return await self._post(
            API_PATH.bitables.batch_delete_table.format(app_token=app_token),
            body=BatchDeleteTableBody(table_ids=table_ids).model_dump(),
            options={"timeout": timeout},
            cast_to=BaseResponse,
        )


class Table(APIResource):
    def list(
        self,
        app_token: str,
        *,
        timeout: Union[httpx.Timeout, None] = None,
        page_token: Union[str, None] = None,
        page_size: Union[int, None] = None,
    ) -> ListTableResponse:
        """根据 app_token，获取多维表格下的所有数据表
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/list

        Args:
            app_token (str): 多维表格的 app_token
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.
            page_token (Union[str, None], optional): 分页标记，第一次请求不填，表示从头开始遍历；\
                分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果。\
                示例值："tblsRc9GRRXKqhvW". Defaults to None.
            
            page_size (Union[int, None], optional): 分页大小，示例值：10，默认值：20，数据校验规则：\
                最大值：100. Defaults to None.

        Returns:
            ListTableResponse: 数据表列表
        """
        return self._get(
            API_PATH.bitables.list_tables.format(app_token=app_token),
            options={
                "timeout": timeout,
                "params": ListTableParams(
                    page_token=page_token,
                    page_size=page_size,
                ).model_dump(),
            },
            cast_to=ListTableResponse,
        )

    def create(
        self,
        app_token: str,
        *,
        table: TableData,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> CreateTableResponse:
        """新增数据表
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/create

        Args:
            app_token (str): 多维表格的 app_token
            table (TableData): 表格数据
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            CreateTableResponse: 创建返回值
        """
        return self._post(
            API_PATH.bitables.create_table.format(app_token=app_token),
            body=CreateTableBody(
                table=table,
            ).model_dump(),
            options={"timeout": timeout},
            cast_to=CreateTableResponse,
        )

    def batch_create(
        self,
        app_token: str,
        *,
        names: List[str],
        user_id_type: Union[Literal["user_id", "open_id", "union_id"], None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BatchCreateTableResponse:
        """新增多个数据表。
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/batch_create

        Args:
            app_token (str): 多维表格的唯一标识符
            names (List[str]): 数据表名称，注意名称中的首尾空格将会被去除。长度范围：1 字符 ～ 100 字符。
            user_id_type (Union[Literal[&quot;user_id&quot;, &quot;open_id&quot;, &quot;union_id&quot;], None], optional): 用户 ID 类型. Defaults to None.
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BatchCreateTableResponse: 创建返回值
        """
        return self._post(
            API_PATH.bitables.batch_create_table.format(app_token=app_token),
            body=BatchCreateTableBody(tables=[{"name": name} for name in names]).model_dump(),
            cast_to=BatchCreateTableResponse,
            options={
                "timeout": timeout,
                "params": BatchCreateTableParams(user_id_type=user_id_type).model_dump(),
            },
        )

    def update(
        self,
        app_token: str,
        *,
        table_id: str,
        name: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> UpdateTableResponse:
        r"""该接口用于更新数据表的基本信息，包括数据表的名称等。

        Args:
            app_token (str): 多维表格的 app_token
            table_id (str): 多维表格的 table_id
            name (str): 数据表的新名称。请注意：名称中的首尾空格将会被去除。
                如果名称为空或和旧名称相同，接口仍然会返回成功，但是名称不会被更改。\n
                示例值："数据表的新名称"。\
                数据校验规则：长度范围：1 字符 ～ 100 字符。正则校验：^[^\[\]\:\\\/\?\*]+$。
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            UpdateTableResponse: 更新返回值
        """
        return self._patch(
            API_PATH.bitables.update_table.format(app_token=app_token, table_id=table_id),
            body=UpdateTableBody(name=name).model_dump(),
            cast_to=UpdateTableResponse,
            options={"timeout": timeout},
        )

    def delete(
        self,
        app_token: str,
        *,
        table_id: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BaseResponse:
        """删除数据表
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/delete

        Args:
            app_token (str): 多维表格的 app_token
            table_id (str): 多维表格的 table_id
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BaseResponse: 删除返回值
        """
        return self._delete(
            API_PATH.bitables.delete_table.format(app_token=app_token, table_id=table_id),
            options={"timeout": timeout},
            cast_to=BaseResponse,
        )

    def batch_delete(
        self,
        app_token: str,
        *,
        table_ids: List[str],
        timeout: Union[httpx.Timeout, None] = None,
    ) -> BaseResponse:
        """批量删除数据表
        https://open.feishu.cn/document/server-docs/docs/bitable-v1/app-table/batch_delete

        Args:
            app_token (str): 多维表格的 app_token
            table_ids (List[str]): 待删除的数据表的 id
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            BaseResponse: 删除返回值
        """
        return self._post(
            API_PATH.bitables.batch_delete_table.format(app_token=app_token),
            body=BatchDeleteTableBody(table_ids=table_ids).model_dump(),
            options={"timeout": timeout},
            cast_to=BaseResponse,
        )



import asyncio
import inspect
import json
import threading
import typing as t

from fastapi.responses import Response
from loguru import logger
from typing_extensions import TYPE_CHECKING

from slark.types.event.common import EventType, LarkEvent
from slark.utils.decrypt import AESCipher

if TYPE_CHECKING:
    from slark.client.lark import AsyncLark


class EventManager:
    _client: "AsyncLark"
    _callback_map: t.Dict[EventType, t.Callable]

    def __init__(self, client: "AsyncLark"):
        self._client = client
        self._callback_map = {}

    def register(self, event_type: EventType, *, run_in_thread: bool = True) -> t.Callable:
        def decorator(func):
            self._callback_map[event_type.value] = {
                "func": func,
                "run_in_thread": run_in_thread,
            }
            return func

        return decorator

    @staticmethod
    def _decrypt_data(encrypt_key, encrypt_data):
        if encrypt_key == "" and encrypt_data is None:
            # data haven't been encrypted
            return None
        if encrypt_key == "":
            raise Exception("ENCRYPT_KEY is necessary")
        cipher = AESCipher(encrypt_key)

        return json.loads(cipher.decrypt_string(encrypt_data))

    def _make_app(self, path: t.Union[str, None] = None):
        import fastapi

        assert self._client._encrypt_key, "ENCRYPT_KEY is necessary"
        assert self._client._verification_token, "VERIFICATION_TOKEN is necessary"

        app = fastapi.FastAPI()

        @app.post(path or "/")
        async def event_handler(data: t.Dict[str, t.Any]):
            decrypt_data = self._decrypt_data(
                self._client._encrypt_key,
                data.get("encrypt"),
            )
            callback_type = decrypt_data.get("type")
            if callback_type == "url_verification":
                return {"challenge": decrypt_data.get("challenge")}
            try:
                lark_event = LarkEvent.model_validate(decrypt_data)
            except Exception as e:
                raise Exception(f"Invalid event data: {decrypt_data}") from e
            event_type = lark_event.header.event_type
            callback = self._callback_map.get(event_type, None)
            if callback is not None:
                func = callback["func"]
                if callback["run_in_thread"]:
                    if inspect.iscoroutinefunction(func):
                        _thread = threading.Thread(
                            target=asyncio.run, args=(func(self._client, lark_event),)
                        )
                    else:
                        _thread = threading.Thread(target=func, args=(self._client, lark_event))
                    _thread.start()
                else:
                    if inspect.iscoroutinefunction(func):
                        return await func(self._client, lark_event)
                    else:
                        return func(self._client, lark_event)
            else:
                logger.warning(f"Unhandled event: {event_type}")
            return Response()

        return app

    def listen(
        self,
        host: str = "127.0.0.1",
        port: int = 8000,
        path: t.Union[str, None] = None,
    ):
        import uvicorn

        app = self._make_app(path)

        uvicorn.run(app, host=host, port=port)



from typing import List, Union

from pydantic import BaseModel

from slark.types.response import BaseResponse

from .common import BlockInfo


class GetAllBlocksResponseData(BaseModel):
    items: List[BlockInfo]
    page_token: Union[str, None] = None
    has_more: bool


class GetAllBlocksResponse(BaseResponse):
    data: GetAllBlocksResponseData



from typing import Dict, List, Union

from typing_extensions import Literal

from ..base import BaseElement, BaseModel
from ..color import Color


class TableHeaderStyle(BaseModel):
    text_align: Union[Literal["left", "center", "right"], None] = None
    text_size: Union[Literal["normal", "heading"], None] = None
    backgroud_style: Union[Literal["none", "grey"]] = None
    text_color: Union[Literal["default", "grey"], None] = None
    bold: Union[bool, None] = None
    lines: Union[int, None] = None


class TableColumnFormat(BaseModel):
    precision: Union[int, None] = None
    symbol: Union[str, None] = None
    seperator: Union[bool, None] = None


class TableColumn(BaseModel):
    name: str
    """自定义列的标记。用于唯一指定行数据对象数组中，需要将数据填充至这一行的具体哪个单元格中。"""
    display_name: Union[str, None] = None
    """在表头展示的列名称。不填或为空则不展示列名称。"""
    width: Union[str, None] = None
    """列宽度。可取值：\n
    auto：自适应内容宽度\n
    自定义宽度：自定义表格的列宽度，如 120px。取值范围是 [80px,600px] 的整数\n
    自定义宽度百分比：自定义列宽度占当前表格画布宽度的百分比（表格画布宽度 = 卡片宽度-卡片左右内边距），如 25%。取值范围是 [1%,100%]"""

    horizontal_align: Union[Literal["left", "center", "right"], None] = None
    """列内数据对齐方式。可选值：\n
    left：左对齐\n
    center：居中对齐\n
    right：右对齐"""

    data_type: Literal["text", "lark_md", "options", "number", "persons", "date", "markdown"] = (
        "text"
    )
    """列数据类型。可选值如下所示。了解不同类型用法，参考 data_type 字段说明一节。\n
    text：不带格式的普通文本。为 data_type 默认值。
    lark_md：支持部分 markdown 格式的文本。飞书 v7.10 及之后版本支持。详情参考普通文本-lark_md 支持的 Markdown 语法
    options：选项标签
    number：数字。默认在单元格中右对齐展示。若选择该数据类型，你可继续在 column 中添加 format 字段，设置数字的格式属性
    persons：人员列表。为用户名称+头像样式
    date：日期时间。需输入 Unix 标准毫秒级时间戳，飞书客户端将按用户本地时区展示日期时间。飞书 v7.6 及之后版本支持
    markdown：支持完整 Markdown 语法的文本内容。详情参考富文本（Markdown）组件。飞书 v7.14 及之后版本支持"""

    format: Union[TableColumnFormat, None] = None
    date_format: Union[
        Literal["YYYY/MM/DD", "YYYY/MM/DD HH:mm", "YYYY-MM-DD", "YYYY-MM-DD HH:mm"], str, None
    ] = None
    """该字段仅当 data_type 为 date 时生效。你可按需选择以下日期时间占位符，并使用任意分隔符组合。\n
    YYYY：年
    MM：月
    DD：日
    HH：小时
    mm：分钟
    ss：秒
    推荐使用以下日期格式。默认按 RFC 3339 标准格式展示日期时间。

    YYYY/MM/DD
    YYYY/MM/DD HH:mm
    YYYY-MM-DD
    YYYY-MM-DD HH:mm
    DD/MM/YYYY
    MM/DD/YYYY"""


class TableCellOption(BaseModel):
    text: str
    color: Color


TableCellType = Union[
    str,
    int,
    float,
    TableCellOption,
    List[str],
]
"""日期时间。需输入 Unix 标准毫秒级时间戳，飞书客户端将按用户本地时区展示日期时间。支持添加 date_format 字段，设置日期的格式属性。默认按 RFC 3339 标准格式展示日期时间。详情参考 date_format 字段说明。
markdown type 支持完整 Markdown 语法的文本内容。详情参考富文本（Markdown）组件。"""


class TableElement(BaseElement):
    tag: str = "table"
    page_size: Union[int, None] = None
    row_height: Union[str, Literal["low", "middle", "high"], None] = None
    header_style: Union[TableHeaderStyle, None] = None
    columns: List[TableColumn]
    rows: List[Dict[str, TableCellType]]



import enum
import hashlib
import warnings
from typing import Union

from fastapi import Request
from typing_extensions import Literal

from slark.types._common import BaseModel

from .callback import CallbackEvent
from .message import MessageEvent

warnings.filterwarnings("ignore", category=UserWarning)

class EventType(enum.Enum):
    IM_MESSAGE_RECEIVE_V1 = "im.message.receive_v1"
    """机器人接收到用户发送的消息后触发此事件。"""
    IM_MESSAGE_MESSAGE_READ_V1 = "im.message.message_read_v1"
    """用户阅读机器人发送的单聊消息后触发此事件。"""
    IM_MESSAGE_RECALLED_V1 = "im.message.recalled_v1"
    """机器人所在会话内的消息被撤回时触发此事件。"""
    IM_MESSAGE_REACTION_CREATED_V1 = "im.message.reaction.created_v1"
    """应用订阅该事件后，消息被添加表情回复时会触发此事件。事件体包含被添加表情回复的消息 message_id、添加表情回复的操作人 ID、表情类型、添加时间等信息。"""
    IM_MESSAGE_REACTION_DELETED_V1 = "im.message.reaction.deleted_v1"
    """应用订阅该事件后，消息被删除表情回复时会触发此事件。事件体包含被删除表情回复的消息 message_id、删除表情回复的操作人 ID、表情类型、添加时间等信息。"""
    IM_CHAT_DISBANDED_V1 = "im.chat.disbanded_v1"
    """群组被解散后触发此事件。"""
    IM_CHAT_UPDATED_V1 = "im.chat.updated_v1"
    """群组配置被修改后触发此事件，包含：群主转移
    群基本信息修改(群头像/群名称/群描述/群国际化名称)
    群权限修改(加人入群权限/群编辑权限/at所有人权限/群分享权限等)
    用户进群"""

    IM_CHAT_MEMBER_USER_ADDED_V1 = "im.chat.member.user.added_v1"
    """新用户进群（包含话题群）触发此事件。"""
    IM_CHAT_MEMBER_USER_DELETED_V1 = "im.chat.member.user.deleted_v1"
    """用户主动退群或被移出群聊时推送事件。"""
    IM_CHAT_MEMBER_USER_WITHDRAWN_V1 = "im.chat.member.user.withdrawn_v1"
    """撤销拉用户进群后触发此事件。"""
    IM_CHAT_MEMBER_BOT_ADDED_V1 = "im.chat.member.bot.added_v1"
    """机器人被用户添加至群聊时触发此事件。"""
    IM_CHAT_MEMBER_BOT_DELETED_V1 = "im.chat.member.bot.deleted_v1"
    """机器人被移出群聊后触发此事件。"""
    P2P_CHAT_CREATE = "p2p_chat_create"
    """首次会话是用户了解应用的重要机会，你可以发送操作说明、配置地址来指导用户开始使用你的应用。"""
    IM_CHAT_ACCESS_EVENT_BOT_P2P_CHAT_ENTERED_V1 = "im.chat.access_event.bot_p2p_chat_entered_v1"
    """用户进入与机器人的会话时触发此事件"""

    CARD_ACTION_TRIGGER = "card.action.trigger"
    """卡片回传交互作用于飞书卡片的 请求回调 交互组件。当终端用户点击飞书卡片上的回传交互组件后，你在开发者后台应用内注册的回调请求地址将会收到 卡片回传交互 回调。该回调包含了用户与卡片之间的交互信息。\
        你的业务服务器接收到回调请求后，需要在 3 秒内响应回调请求，声明通过弹出 Toast 提示、更新卡片、保持原内容不变等方式响应用户交互。"""


class LarkEventHeader(BaseModel):
    event_id: str
    event_type: EventType
    create_time: str
    token: str
    app_id: str
    tenant_key: str


class InvalidEventException(Exception):
    def __init__(self, error_info):
        self.error_info = error_info

    def __str__(self) -> str:
        return "Invalid event: {}".format(self.error_info)

    __repr__ = __str__


class LarkEvent(BaseModel):
    schema: Literal["2.0"]
    header: LarkEventHeader
    event: Union[MessageEvent, CallbackEvent]

    def _validate(self, request: Request, token: str, encrypt_key: str):
        if self.header.token != token:
            raise InvalidEventException("Invalid token")
        timestamp = request.headers.get("X-Lark-Request-Timestamp")
        nonce = request.headers.get("X-Lark-Request-Nonce")
        signature = request.headers.get("X-Lark-Signature")
        body = request._body
        bytes_b1 = (timestamp + nonce + encrypt_key).encode("utf-8")
        bytes_b = bytes_b1 + body
        h = hashlib.sha256(bytes_b)
        if signature != h.hexdigest():
            raise InvalidEventException("invalid signature in event")



from typing import Union

from slark.types._common import BaseModel

from ..common import FieldType, UIType
from .common import FieldDescription, FieldPropertyType


class ListFieldParams(BaseModel):
    view_id: Union[str, None] = None
    """视图 ID

    示例值："vewOVMEXPF"
    """

    text_field_as_array: Union[bool, None] = None
    """控制字段描述（多行文本格式）数据的返回格式, true 表示以数组富文本形式返回

    示例值：true"""

    page_token: Union[str, None] = None
    """分页标记，第一次请求不填，表示从头开始遍历；分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果

    示例值："fldwJ4YrtB"
    """

    page_size: Union[int, None] = None
    """	
    分页大小

    示例值：10

    默认值：20

    数据校验规则：

    最大值：100"""


class CreateFieldParams(BaseModel):
    client_token: Union[str, None] = None
    """格式为标准的 uuidv4，操作的唯一标识，用于幂等的进行更新操作。此值为空表示将发起一次新的请求，此值非空表示幂等的进行更新操作。\
        示例值："fe599b60-450f-46ff-b2ef-9f6675625b97"。"""


class CreateFieldBody(BaseModel):
    field_name: str
    """多维表格字段名

    请注意：

    名称中的首尾空格将会被去除。
    示例值："多行文本"
    """
    type: FieldType
    property: Union[FieldPropertyType, None] = None
    """字段属性，因字段类型而异"""
    ui_type: UIType
    description: Union[FieldDescription, None] = None
    """字段描述"""


class UpdateFieldBody(CreateFieldBody):
    pass



from time import time

from slark.types._common import BaseModel


class TokenBase(BaseModel):
    access_token: str
    """访问token"""
    expires_at: int
    """过期时间，单位为秒(从1970年1月1日0时0分0秒开始计算)"""

    @property
    def is_expired(self):
        return time() > self.expires_at

    def refresh(self):
        pass


class UserAccessToken(TokenBase):
    refresh_token: str
    """刷新user_access_token时使用的 refresh_token"""
    token_type: str
    """	token 类型，固定值"""
    refresh_expires_at: int
    """refresh_token有效期，单位: 秒，一般是30天左右，需要以返回结果为准"""
    scope: str
    """用户授予app的权限全集"""


class TenantAccessToken(TokenBase):
    pass



import mimetypes
from typing import Union

import anyio
import httpx
from httpx import Response

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.api_path import API_PATH
from slark.types.drive.file.response import FileInfo
from slark.utils import extract_filename


class AsyncFile(AsyncAPIResource):
    async def download(
        self,
        file_token: str,
        *,
        save_dir: str,
        range: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> FileInfo:
        """下载云空间中的文件，如 PDF 文件。不包含飞书文档、电子表格以及多维表格等在线文档。该接口支持通过在请求头添加 Range 参数分片下载部分文件。
        调用此接口之前，你需确保应用已拥有文件的下载权限。否则接口将返回 403 的 HTTP 状态码。
        本接口仅支持下载云空间中的资源文件。要下载云文档中的素材（如图片、附件等），需调用下载素材接口。
        
        https://open.feishu.cn/document/server-docs/docs/drive-v1/download/download
        
        Args:
            file_token (str): 文件的 token。
            save_dir (str): 保存文件的目录。
            range (str): 在 HTTP 请求头中，通过指定 Range 来下载文件的部分内容，单位是字节（byte）。\
                该参数的格式为 Range: bytes=start-end.\\
                示例值为 Range: bytes=0-1024，表示下载第 0 个字节到第 1024 个字节之间的数据。
            timeout (Union[httpx.Timeout, None], optional): 超时时间。 Defaults to None.

        Returns:
            response: \
                Content-Type 文件的MIME，比如：application/vnd.openxmlformats-officedocument.wordprocessingml.document
                Content-Disposition: 文件名
        """
        response: Response = await self._get(
            API_PATH.drive.file.download.format(file_token=file_token),
            options={
                "timeout": timeout,
                "headers": {"Range": range} if range else {},
                "raw_response": True,
            },
            cast_to=Response,
        )
        mime_type = response.headers["Content-Type"]
        ext = mimetypes.guess_extension(mime_type)
        file_name = response.headers["Content-Disposition"]
        file_name = extract_filename(file_name)

        save_dir_path = anyio.Path(save_dir)
        await save_dir_path.mkdir(exist_ok=True)
        file_path = save_dir_path / f"{file_token}{ext}"

        f = await anyio.open_file(file_path, "wb")
        await f.write(response.content)
        await f.aclose()

        return FileInfo(
            file_path=file_path.as_posix(),
            file_token=file_token,
            file_name=file_name,
            mime_type=mime_type,
        )


class File(APIResource):
    def download(
        self,
        file_token: str,
        *,
        save_dir: str,
        range: Union[str, None] = None,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> FileInfo:
        """下载云空间中的文件，如 PDF 文件。不包含飞书文档、电子表格以及多维表格等在线文档。该接口支持通过在请求头添加 Range 参数分片下载部分文件。
        调用此接口之前，你需确保应用已拥有文件的下载权限。否则接口将返回 403 的 HTTP 状态码。
        本接口仅支持下载云空间中的资源文件。要下载云文档中的素材（如图片、附件等），需调用下载素材接口。
        
        https://open.feishu.cn/document/server-docs/docs/drive-v1/download/download
        
        Args:
            file_token (str): 文件的 token。
            save_dir (str): 保存文件的目录。
            range (str): 在 HTTP 请求头中，通过指定 Range 来下载文件的部分内容，单位是字节（byte）。\
                该参数的格式为 Range: bytes=start-end.\\
                示例值为 Range: bytes=0-1024，表示下载第 0 个字节到第 1024 个字节之间的数据。
            timeout (Union[httpx.Timeout, None], optional): 超时时间。 Defaults to None.

        Returns:
            response: \
                Content-Type 文件的MIME，比如：application/vnd.openxmlformats-officedocument.wordprocessingml.document
                Content-Disposition: 文件名
        """
        response: Response = self._get(
            API_PATH.drive.file.download.format(file_token=file_token),
            options={
                "timeout": timeout,
                "headers": {"Range": range} if range else {},
                "raw_response": True,
            },
            cast_to=Response,
        )
        mime_type = response.headers["Content-Type"]
        ext = mimetypes.guess_extension(mime_type)
        file_name = response.headers["Content-Disposition"]
        file_name = extract_filename(file_name)

        save_dir_path = anyio.Path(save_dir)
        save_dir_path.mkdir(exist_ok=True)
        file_path = save_dir_path / f"{file_token}{ext}"

        with anyio.open_file(file_path, "wb") as f:
            f.write(response.content)

        return FileInfo(
            file_path=file_path.as_posix(),
            file_token=file_token,
            file_name=file_name,
            mime_type=mime_type,
        )



from typing import List, Union

from slark.types._common import BaseModel

from .common import TableData


class CreateTableBody(BaseModel):
    table: TableData


class BatchCreateTableData(BaseModel):
    name: str


class BatchCreateTableBody(BaseModel):
    tables: List[BatchCreateTableData]


class BatchCreateTableParams(BaseModel):
    user_id_type: Union[str, None] = None


class BatchDeleteTableBody(BaseModel):
    table_ids: List[str]
    """待删除的数据表的id [table_id 参数说明]，当前一次操作最多支持50个数据表(/ssl:ttdoc/uAjLw4CM/ukTMukTMukTM/bitable/notification#735fe883)

    示例值：["tbl1TkhyTWDkSoZ3"]"""


class UpdateTableBody(BaseModel):
    name: str
    r"""	
    数据表的新名称。

    请注意：

    名称中的首尾空格将会被去除。

    如果名称为空或和旧名称相同，接口仍然会返回成功，但是名称不会被更改。

    示例值："数据表的新名称"

    数据校验规则：

    长度范围：1 字符 ～ 100 字符
    正则校验：^[^\[\]\:\\\/\?\*]+$"""


class ListTableParams(BaseModel):
    page_token: Union[str, None] = None
    """分页标记，第一次请求不填，表示从头开始遍历；分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果

    示例值：tblsRc9GRRXKqhvW"""
    page_size: Union[int, None] = None
    """分页大小

    示例值：10

    默认值：20

    数据校验规则：

    最大值：100"""



import re
from urllib.parse import unquote


def extract_filename(disposition: str):
    match = re.search(r"filename\*=([^;]+)", disposition)
    if match:
        try:
            enc_quoted = match.group(1).strip().strip('"')
            enc, quoted = enc_quoted.split("''")
            return unquote(quoted, encoding=enc)
        except Exception:
            return "Unkown"
    else:
        return "Unknown"


from slark.types.response import BaseResponse


class GetTenantAccessTokenResponse(BaseResponse):
    tenant_access_token: str
    """租户访问凭证"""
    expire: int
    """tenant_access_token 的过期时间，单位为秒"""



from typing import Any, Union

import httpx
from requests_toolbelt import MultipartEncoder
from typing_extensions import Literal

from slark.resources._resources import AsyncAPIResource, APIResource
from slark.resources.api_path import API_PATH
from slark.types.messages.image.response import UploadImageResponse


class AsyncImage(AsyncAPIResource):
    async def upload(
        self,
        image: Any,
        image_type: Literal["message", "avatar"] = "message",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> UploadImageResponse:
        """上传图片

        Args:
            image_type (str): 图片类型\
                示例值："message"\
                可选值有：\
                    message：用于发送消息\
                    avatar：用于设置头像\
            
            image (Any): 图片内容。传值方式可以参考请求体示例。\
                注意：\
                上传的图片大小不能超过 10 MB，也不能上传大小为 0 的图片。\
                分辨率限制：\
                GIF 图片分辨率不能超过 2000 x 2000，其他图片分辨率不能超过 12000 x 12000。\
                用于设置头像的图片分辨率不能超过 4096 x 4096。\
                示例值：二进制文件\
                
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            UploadImageResponse: 返回值
        """
        m = MultipartEncoder(
            fields={
                "image_type": image_type,
                "image": ("image", image),
            }
        )
        return await self._post(
            API_PATH.image.upload,
            cast_to=UploadImageResponse,
            options={
                "content": m.read(),
                "timeout": timeout,
                "headers": {
                    "Content-Type": m.content_type,
                },
            },
        )


class Image(APIResource):
    def upload(
        self,
        image: Any,
        image_type: Literal["message", "avatar"] = "message",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> UploadImageResponse:
        """上传图片

        Args:
            image_type (str): 图片类型\
                示例值："message"\
                可选值有：\
                    message：用于发送消息\
                    avatar：用于设置头像\
            
            image (Any): 图片内容。传值方式可以参考请求体示例。\
                注意：\
                上传的图片大小不能超过 10 MB，也不能上传大小为 0 的图片。\
                分辨率限制：\
                GIF 图片分辨率不能超过 2000 x 2000，其他图片分辨率不能超过 12000 x 12000。\
                用于设置头像的图片分辨率不能超过 4096 x 4096。\
                示例值：二进制文件\
                
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            UploadImageResponse: 返回值
        """
        m = MultipartEncoder(
            fields={
                "image_type": image_type,
                "image": ("image", image),
            }
        )
        return self._post(
            API_PATH.image.upload,
            cast_to=UploadImageResponse,
            options={
                "content": m.read(),
                "timeout": timeout,
                "headers": {
                    "Content-Type": m.content_type,
                },
            },
        )



from typing import Union

from slark.types._common import BaseModel
from slark.types.bitables.common import FieldPropertyType, FieldType, UIType


class FieldDescription(BaseModel):
    disable_sync: bool
    """是否禁止同步，如果为true，表示禁止同步该描述内容到表单的问题描述注意：该字段在列出字段时不生效。只在新增、修改字段时生效。"""
    text: str
    """字段描述内容"""


class Field(BaseModel):
    field_id: str
    """字段id"""

    field_name: str
    """字段名"""

    type: FieldType

    property: Union[FieldPropertyType, None] = None
    """字段属性，因字段类型而异"""

    ui_type: Union[UIType, None] = None

    description: Union[FieldDescription, None] = None
    """字段描述"""

    is_primary: Union[bool, None] = None
    """是否是索引列"""

    is_hidden: Union[bool, None] = None
    """是否是隐藏字段"""



from typing import Union

from .base import BaseElement
from .color import Color
from .ud_icon import UDIconToken


class IconElement(BaseElement):
    tag: str = "standard_icon"
    """图标类型的标签。可取值：

    standard_icon：使用图标库中的图标。
    custom_icon：使用用自定义图片作为图标。"""

    token: Union[UDIconToken, None] = None
    """图标库中图标的 token。当 tag 为 standard_icon 时生效。枚举值参见图标库。"""

    color: Union[Color, None] = None
    """图标的颜色。支持设置线性和面性图标（即 token 末尾为 outlined 或 filled 的图标）的颜色。当 tag 为 standard_icon 时生效。枚举值参见颜色枚举值。"""

    img_key: Union[str, None] = None
    """自定义前缀图标的图片 key。当 tag 为 custom_icon 时生效。

    图标 key 的获取方式：调用上传图片接口，上传用于发送消息的图片，并在返回值中获取图片的 image_key。"""



import os
from typing import Union

import httpx
from loguru import logger

from slark import resources
from slark._constants import DEFAULT_MAX_RETRIES, DEFAULT_TIMEOUT
from slark.client._sync_client import APIClient
from slark.types.auth import CredentailTypes, TokenBase


class Lark(APIClient):
    auth: resources.Auth
    webhook: resources.Webhook
    knowledge_space: resources.KnowledgeSpace
    sheets: resources.Spreadsheets
    bitables: resources.BiTable
    docs: resources.Documents
    board: resources.Board
    assets: resources.Assets
    messages: resources.Messages
    drive: resources.Drive

    _app_id: Union[str, None]
    _app_secret: Union[str, None]
    _webhook_url: Union[str, None]
    _token: Union[TokenBase, None]
    _token_type: CredentailTypes

    def __init__(
        self,
        *,
        app_id: Union[str, None] = None,
        app_secret: Union[str, None] = None,
        webhook: Union[str, None] = None,
        verification_token: Union[str, None] = None,
        encrypt_key: Union[str, None] = None,
        base_url: Union[str, httpx.URL] = "https://open.feishu.cn/open-apis/",
        max_retries: int = DEFAULT_MAX_RETRIES,
        timeout: httpx.Timeout = DEFAULT_TIMEOUT,
        proxies: Union[httpx._types.ProxyTypes, None] = None,
        token_type: CredentailTypes = "tenant",
    ):
        self._app_id = app_id or os.getenv("APP_ID", None)
        self._app_secret = app_secret or os.getenv("APP_SECRET", None)
        self._webhook_url = webhook or os.getenv("WEBHOOK_URL", None)
        self._verification_token = verification_token or os.getenv("VERIFICATION_TOKEN", None)
        self._encrypt_key = encrypt_key or os.getenv("ENCRYPT_KEY", None)
        self._token = None
        self._token_type = token_type

        super().__init__(
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            proxies=proxies,
        )

        self.auth = resources.Auth(self)
        self.drive = resources.Drive(self)
        self.webhook = resources.Webhook(self)
        self.knowledge_space = resources.KnowledgeSpace(self)
        self.sheets = resources.Spreadsheets(self)
        self.bitables = resources.BiTable(self)
        self.docs = resources.Documents(self)
        self.board = resources.Board(self)
        self.assets = resources.Assets(self)
        self.messages = resources.Messages(self)

    @property
    def app_credentials(self) -> dict:
        if self._app_id is None or self._app_secret is None:
            raise ValueError("App credentials are not set")
        return {
            "app_id": self._app_id,
            "app_secret": self._app_secret,
        }

    def get_auth_headers(self) -> dict:
        if self._token is None or self._token.is_expired:
            if self._token_type == "tenant":
                logger.debug("Refreshing tenant access token")
                self._token = self.auth.token.get_tenant_access_token()
            else:
                raise NotImplementedError(f"{self._token_type} token is not supported")
        return {
            "Authorization": f"Bearer {self._token.access_token}",
        }



from typing import List, Union

from typing_extensions import Literal

from ..base import BaseElement, BaseModel
from ..displaying.chart import ChartElement
from ..displaying.divider import HlineElement
from ..displaying.image import ImageElement
from ..displaying.image_combination import ImageCombinationElement
from ..displaying.markdown import MDElement
from ..displaying.note import NoteElement
from ..displaying.person import PersonElement
from ..displaying.person_list import PersonListElement
from ..displaying.plain_text import DivElement
from ..interactive.button import ButtonElement
from ..interactive.checker import CheckerElement
from ..interactive.date_picker import DatePickerElement
from ..interactive.input_box import InputBoxElement
from ..interactive.multi_select_person import MultiSelectPersonElement
from ..interactive.multi_select_static import MultiSelectStaticElement
from ..interactive.picker_datetime import PickerDatetimeElement
from ..interactive.picker_time import PickerTimeElement
from ..interactive.select_img import SelectImgElement
from ..interactive.select_person import SelectPersonElement
from ..interactive.select_static import SelectStaticElement

"""https://open.feishu.cn/document/uAjLw4CM/ukzMukzMukzM/feishu-cards/card-components/containers/column-set"""


class ColumnSetActionMultiUrl(BaseModel):
    url: Union[str, None] = None
    android_url: Union[str, None] = None
    ios_url: Union[str, None] = None
    pc_url: Union[str, None] = None


class ColumnSetAction(BaseModel):
    multi_url: Union[ColumnSetActionMultiUrl, None] = None


ColumnSupportElementType = Union[
    MDElement,
    DivElement,
    ImageElement,
    HlineElement,
    "ColumnSetElement",
    ChartElement,
    ImageCombinationElement,
    PersonElement,
    PersonListElement,
    NoteElement,
    ButtonElement,
    InputBoxElement,
    MultiSelectPersonElement,
    MultiSelectStaticElement,
    SelectPersonElement,
    SelectStaticElement,
    SelectImgElement,
    PickerDatetimeElement,
    PickerTimeElement,
    DatePickerElement,
    CheckerElement,
]


class ColumnElement(BaseElement):
    tag: str = "column"
    background_style: str = "default"
    """列的背景色样式。可取值：

    default：默认的白底样式，客户端深色主题下为黑底样式
    卡片支持的颜色枚举值和 RGBA 语法自定义颜色。参考颜色枚举值。"""

    elements: List[ColumnSupportElementType] = []
    """列容器内嵌的组件，不支持内嵌表格组件、多图混排组件和表单容器。"""

    width: Union[Literal["auto", "weighted"], str] = "auto"
    """列宽度。仅 flex_mode 为 none 时，生效此属性。取值：

    auto：列宽度与列内元素宽度一致
    weighted：列宽度按 weight 参数定义的权重分布
    具体数值，如 100px。取值范围为 [16,600]px。V7.4 及以上版本支持该枚举"""

    weight: int = 1
    """当 width 字段取值为 weighted 时生效，表示当前列的宽度占比。取值范围为 1 ~ 5 之间的整数。"""

    vertical_align: Literal["top", "center", "bottom"] = "top"
    """列垂直居中的方式。可取值：

    top：上对齐
    center：居中对齐
    bottom：下对齐"""

    vertical_spacing: Union[Literal["default", "medium", "large"], str] = "default"
    """列内组件的纵向间距。取值：

    default：默认间距，8px
    medium：中等间距
    large：大间距
    具体数值，如 8px。取值范围为 [0,28]px"""

    padding: Union[str, None] = None
    """列的内边距。值的取值范围为 [0,28]px。可选值：

    单值，如 "10px"，表示列的四个外边距都为 10 px。
    多值，如 "4px 12px 4px 12px"，表示列的上、右、下、左的外边距分别为 4px，12px，4px，12px。四个值必填，使用空格间隔。"""

    action: Union[ColumnSetAction, None] = None


class ColumnSetElement(BaseElement):
    tag: str = "column_set"
    """组件的标签。分栏组件的固定值为 column_set。"""
    flex_mode: Literal["none", "stretch", "flow", "bisect", "trisect"] = "none"
    """移动端和 PC 端的窄屏幕下，各列的自适应方式。取值：
    none：不做布局上的自适应，在窄屏幕下按比例压缩列宽度
    stretch：列布局变为行布局，且每列（行）宽度强制拉伸为 100%，所有列自适应为上下堆叠排布
    flow：列流式排布（自动换行），当一行展示不下一列时，自动换至下一行展示
    bisect：两列等分布局
    trisect：三列等分布局"""

    horizontal_spacing: Union[Literal["default", "small", "large"], str] = "default"
    """各列之间的水平分栏间距。取值：
    default：默认间距，8px
    small：窄间距，4px
    large：大间距，12px
    [0, 28px]：自定义间距"""

    horizontal_align: Literal["left", "center", "right"] = "left"
    """列容器水平对齐的方式。可取值：
    left：左对齐
    center：居中对齐
    right：右对齐"""

    margin: Union[str, None] = None
    """列的外边距。值的取值范围为 [0,28]px。可选值：
    单值，如 "10px"，表示列的四个外边距都为 10 px。
    多值，如 "4px 12px 4px 12px"，表示列的上、右、下、左的外边距分别为 4px，12px，4px，12px。四个值必填，使用空格间隔。
    注意：首行列的上外边距强制为 0，末行列的下外边距强制为 0。"""

    background_style: str = "default"
    """分栏的背景色样式。可取值：
    default：默认的白底样式，客户端深色主题下为黑底样式
    卡片支持的颜色枚举值和 RGBA 语法自定义颜色。参考颜色枚举值。
    注意：当存在分栏的嵌套时，上层分栏的颜色覆盖下层分栏的颜色。"""

    columns: List[ColumnElement] = []
    """分栏中列的配置。"""

    action: Union[ColumnSetAction, None] = None



from typing import List

import pandas as pd

from slark.types.spreadsheets.data import CellTypes


def values_to_dataframe(
    values: List[List[CellTypes]], *, has_header: bool = True, dropna: bool = True
) -> pd.DataFrame:
    values = [[cell.text if hasattr(cell, "text") else cell for cell in row] for row in values]
    if has_header:
        df = pd.DataFrame(values[1:], columns=values[0])
    else:
        df = pd.DataFrame(values)
    if dropna:
        df = df.dropna(axis=0, how="all").dropna(axis=1, how="all")
    return df


def dataframe_to_values(df: pd.DataFrame, *, has_header: bool = True) -> List[List[CellTypes]]:
    if has_header:
        values = [df.columns.tolist()] + df.values.tolist()
    else:
        values = df.values.tolist()
    return values



from .base import BaseElement
from .behavior import BehaviorType, CallbackBehavior, FormActionBehavior, OpenURLBehavior
from .card import (
    CardConfig,
    CardConfigStyle,
    CardConfigStyleColor,
    CardConfigStyleTextSize,
    CardLink,
    ElementType,
    I18nBody,
    I18nHeader,
    I18nHeaderElement,
    InteractiveCard,
)
from .color import Color
from .confirm import ConfirmDialogue
from .container.collapsible import CollapsibleElement
from .container.column import (
    ColumnElement,
    ColumnSetAction,
    ColumnSetActionMultiUrl,
    ColumnSetElement,
    ColumnSupportElementType,
)
from .container.form import FormElement, FormElementType
from .container.interactive_container import InteractiveContainerElement
from .displaying.chart import ChartElement
from .displaying.divider import HlineElement
from .displaying.header import (
    CardHeader,
    CardHeaderI18nTextTag,
    CardHeaderIcon,
    CardHeaderSubtitle,
    CardHeaderTextTag,
    CardHeaderTextTagText,
    CardHeaderTitle,
    CardTemplate,
    I18nText,
    TextTagColor,
)
from .displaying.image import ImageElement, ImageTextElement
from .displaying.image_combination import ImageCombinationElement, ImageCombinationList
from .displaying.markdown import MDElement, MDHref, MDHrefUrlVal
from .displaying.note import NoteElement
from .displaying.person import PersonElement
from .displaying.person_list import PersonListElement, PersonListPerson
from .displaying.plain_text import DivElement, PlainTextElement
from .displaying.table import (
    TableCellOption,
    TableCellType,
    TableColumn,
    TableColumnFormat,
    TableElement,
    TableHeaderStyle,
)
from .icon import IconElement
from .interactive.action import ActionElements, ActionElementType
from .interactive.button import ButtonElement
from .interactive.checker import CheckerElement
from .interactive.date_picker import DatePickerElement
from .interactive.input_box import InputBoxElement, InputBoxFallback
from .interactive.multi_select_person import MultiSelectPersonElement
from .interactive.multi_select_static import MultiSelectStaticElement
from .interactive.overflow import OverflowElement
from .interactive.picker_datetime import PickerDatetimeElement
from .interactive.picker_time import PickerTimeElement
from .interactive.select_img import SelectImgElement
from .interactive.select_person import SelectPersonElement
from .interactive.select_static import SelectStaticElement
from .text import PlainText, TextSize
from .ud_icon import UDIconElement, UDIconStyle, UDIconToken

__all__ = [
    "ActionElementType",
    "ActionElements",
    "BaseElement",
    "Color",
    "CardHeader",
    "CardHeaderI18nTextTag",
    "CardHeaderIcon",
    "CardHeaderSubtitle",
    "CardHeaderTitle",
    "CardHeaderTextTag",
    "CardHeaderTextTagText",
    "CardTemplate",
    "TextTagColor",
    "I18nText",
    "UDIconElement",
    "UDIconStyle",
    "UDIconToken",
    "CardConfig",
    "CardConfigStyle",
    "CardConfigStyleColor",
    "CardConfigStyleTextSize",
    "CardHeader",
    "CardLink",
    "ChartElement",
    "ColumnElement",
    "ColumnSetAction",
    "ColumnSetActionMultiUrl",
    "ColumnSetElement",
    "ColumnSupportElementType",
    "ImageCombinationElement",
    "I18nBody",
    "I18nHeader",
    "I18nHeaderElement",
    "IconElement",
    "ImageCombinationList",
    "ImageElement",
    "ImageTextElement",
    "InteractiveCard",
    "TableCellOption",
    "TableCellType",
    "TableColumn",
    "TableColumnFormat",
    "TableElement",
    "TableHeaderStyle",
    "TextSize",
    "PlainTextElement",
    "FormElementType",
    "DivElement",
    "FormElement",
    "NoteElement",
    "HlineElement",
    "PersonElement",
    "PersonListElement",
    "PersonListPerson",
    "MDElement",
    "MDHrefUrlVal",
    "MDHref",
    "ElementType",
    "InputBoxText",
    "InputBoxElement",
    "InputBoxConfirm",
    "InputBoxFallback",
    "ButtonElement",
    "ButtonConfirm",
    "ButtonText",
    "CallbackBehavior",
    "FormActionBehavior",
    "OpenURLBehavior",
    "BehaviorType",
    "ConfirmDialogue",
    "OverflowElement",
    "PickerDatetimeElement",
    "PickerTimeElement",
    "DatePickerElement",
    "CheckerElement",
    "SelectImgElement",
    "SelectPersonElement",
    "SelectStaticElement",
    "MultiSelectPersonElement",
    "MultiSelectStaticElement",
    "CheckerElement",
    "InteractiveContainerElement",
    "CollapsibleElement",
    "ColumnSetElement",
    "PlainText",
]



import re
from typing import Union

import anyio
import httpx
from httpx import Response
from pydantic import BaseModel

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.api_path import API_PATH


class DownloadFileInfo(BaseModel):
    filepath: str
    filename: str
    filetype: str


class AsyncAssets(AsyncAPIResource):
    async def download(
        self,
        file_token: str,
        *,
        save_dir: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> DownloadFileInfo:
        response = await self._get(
            API_PATH.assets.download.format(file_token=file_token),
            options={
                "timeout": timeout,
                "raw_response": True,
            },
            cast_to=Response,
        )
        mime_type = response.headers["Content-Type"]
        filename = re.findall(r'filename="(.+)"', response.headers["Content-Disposition"])[0]
        filename = f"{file_token}_{filename}"
        path = anyio.Path(save_dir) / filename
        await path.parent.mkdir(parents=True, exist_ok=True)
        f = await anyio.open_file(path, "wb")
        await f.write(response.content)
        await f.aclose()
        return DownloadFileInfo(
            filepath=path.as_posix(),
            filename=filename,
            filetype=mime_type,
        )


class Assets(APIResource):
    def download(
        self,
        file_token: str,
        *,
        save_dir: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> DownloadFileInfo:
        response = self._get(
            API_PATH.assets.download.format(file_token=file_token),
            options={
                "timeout": timeout,
                "raw_response": True,
            },
            cast_to=Response,
        )
        mime_type = response.headers["Content-Type"]
        filename = re.findall(r'filename="(.+)"', response.headers["Content-Disposition"])[0]
        filename = f"{file_token}_{filename}"
        path = anyio.Path(save_dir) / filename
        path.parent.mkdir(parents=True, exist_ok=True)

        with path.open("wb") as f:
            f.write(response.content)
        return DownloadFileInfo(
            filepath=path.as_posix(),
            filename=filename,
            filetype=mime_type,
        )



from .common import CellTypes
from .range import Range
from .request import (
    AppendDataBody,
    AppendDataParams,
    PrependDataBody,
    ReadMultiRangeParams,
    ReadSingleRangeParams,
    WriteDataValueRange,
    WriteMultiRangeBody,
    WriteSingleRangeBody,
)
from .response import (
    AppendDataResponse,
    PrependDataResponse,
    ReadMultiRangeResponse,
    ReadSingleRangeResponse,
    WriteMultiRangeResponse,
    WriteSingleRangeResponse,
)

__all__ = [
    "Range",
    "CellTypes",
    "PrependDataBody",
    "AppendDataBody",
    "AppendDataParams",
    "ReadMultiRangeParams",
    "ReadSingleRangeParams",
    "WriteMultiRangeBody",
    "WriteSingleRangeBody",
    "WriteDataValueRange",
    "PrependDataResponse",
    "AppendDataResponse",
    "ReadMultiRangeResponse",
    "ReadSingleRangeResponse",
    "WriteMultiRangeResponse",
    "WriteSingleRangeResponse",
]



from typing import Union

from typing_extensions import Literal

from slark.types._common import BaseModel


class GetAllBlocksParams(BaseModel):
    page_size: Union[int, None] = None
    """分页大小,默认值/最大值：500"""
    page_token: Union[str, None] = None
    """分页标记，第一次请求不填，表示从头开始遍历；\
        分页查询结果还有更多项时会同时返回新的 page_token，下次遍历可采用该 page_token 获取查询结果"""
    document_revision_id: Union[int, None] = None
    """查询的文档版本，-1 表示文档最新版本。文档创建后，版本为 1。\
        若查询的版本为文档最新版本，则需要持有文档的阅读权限；若查询的版本为文档的历史版本，则需要持有文档的编辑权限。"""
    user_id_type: Union[Literal["open_id", "union_id", "user_id"], None] = None
    """用户 ID 类型。\
        示例值："open_id"。\
        可选值有：\n
        open_id：标识一个用户在某个应用中的身份。同一个用户在不同应用中的 Open ID 不同。了解更多：如何获取 Open ID\n
        union_id：标识一个用户在某个应用开发商下的身份。\
            同一用户在同一开发商下的应用中的 Union ID 是相同的，在不同开发商下的应用中的 Union ID 是不同的。通过 Union ID，\
            应用开发商可以把同个用户在多个应用中的身份关联起来。了解更多：如何获取 Union ID？\n
        user_id：标识一个用户在某个租户内的身份。\
            同一个用户在租户 A 和租户 B 内的 User ID 是不同的。\
            在同一个租户内，一个用户的 User ID 在所有应用（包括商店应用）中都保持一致。\
            User ID 主要用于在不同的应用间打通用户数据。了解更多：如何获取 User ID？"""



from slark.types._common import BaseModel


class GetTenantAccessTokenParams(BaseModel):
    app_id: str
    """应用唯一标识，创建应用后获得。有关app_id 的详细介绍。
    请参考通用参数介绍 https://open.feishu.cn/document/server-docs/api-call-guide/terminology"""
    app_secret: str
    """应用秘钥，创建应用后获得。有关 app_secret 的详细介绍，请参考通用参数介绍"""



from slark.types._common import BaseModel
from slark.types.response.base import BaseResponse


class UploadImageData(BaseModel):
    image_key: str
    """图片key，用于后续获取图片信息"""


class UploadImageResponse(BaseResponse):
    data: UploadImageData



from typing import Union

import httpx
from typing_extensions import Literal

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.api_path import API_PATH
from slark.types.spreadsheets import GetSpreadsheetInfoResponse


class AsyncTable(AsyncAPIResource):
    async def get_spreadsheet_info(
        self,
        sheet_token: str,
        *,
        user_id_type: Literal["open_id", "union_id", "user_id"] = "open_id",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetSpreadsheetInfoResponse:
        """根据电子表格 token 获取电子表格的基础信息，包括电子表格的所有者、URL 链接等。
        https://open.feishu.cn/document/server-docs/docs/sheets-v3/spreadsheet/get
        Args:
            sheet_token (str): 电子表格的 token。
            user_id_type (Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], optional): 用户 ID 类型
                示例值："open_id"
                可选值有：
                open_id：标识一个用户在某个应用中的身份。同一个用户在不同应用中的 Open ID 不同。
                union_id：标识一个用户在某个应用开发商下的身份。同一用户在同一开发商下的应用中的 Union ID 是相同的，在不同开发商下的应用中的 Union ID 是不同的。通过 Union ID，应用开发商可以把同个用户在多个应用中的身份关联起来。
                user_id：标识一个用户在某个租户内的身份。同一个用户在租户 A 和租户 B 内的 User ID 是不同的。在同一个租户内，一个用户的 User ID 在所有应用（包括商店应用）中都保持一致。User ID 主要用于在不同的应用间打通用户数据。
                默认值：open_id。当值为 user_id，字段权限要求：获取用户 user ID.

        Returns:
            GetSpreadsheetInfoResponse: 电子表格的基础信息
        """
        return await self._get(
            API_PATH.spreadsheets.get_spreadsheet_info.format(spreadsheet_token=sheet_token),
            cast_to=GetSpreadsheetInfoResponse,
            options={"timeout": timeout, "params": {"user_id_type": user_id_type}},
        )


class Table(APIResource):
    def get_spreadsheet_info(
        self,
        sheet_token: str,
        *,
        user_id_type: Literal["open_id", "union_id", "user_id"] = "open_id",
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetSpreadsheetInfoResponse:
        """根据电子表格 token 获取电子表格的基础信息，包括电子表格的所有者、URL 链接等。
        https://open.feishu.cn/document/server-docs/docs/sheets-v3/spreadsheet/get
        Args:
            sheet_token (str): 电子表格的 token。
            user_id_type (Literal[&quot;open_id&quot;, &quot;union_id&quot;, &quot;user_id&quot;], optional): 用户 ID 类型
                示例值："open_id"
                可选值有：
                open_id：标识一个用户在某个应用中的身份。同一个用户在不同应用中的 Open ID 不同。
                union_id：标识一个用户在某个应用开发商下的身份。同一用户在同一开发商下的应用中的 Union ID 是相同的，在不同开发商下的应用中的 Union ID 是不同的。通过 Union ID，应用开发商可以把同个用户在多个应用中的身份关联起来。
                user_id：标识一个用户在某个租户内的身份。同一个用户在租户 A 和租户 B 内的 User ID 是不同的。在同一个租户内，一个用户的 User ID 在所有应用（包括商店应用）中都保持一致。User ID 主要用于在不同的应用间打通用户数据。
                默认值：open_id。当值为 user_id，字段权限要求：获取用户 user ID.

        Returns:
            GetSpreadsheetInfoResponse: 电子表格的基础信息
        """
        return self._get(
            API_PATH.spreadsheets.get_spreadsheet_info.format(spreadsheet_token=sheet_token),
            cast_to=GetSpreadsheetInfoResponse,
            options={"timeout": timeout, "params": {"user_id_type": user_id_type}},
        )



from typing_extensions import Literal

from slark.types._common import BaseModel

NodeTypes = Literal["doc", "docx", "sheet", "mindnote", "bitable", "file", "slides", "wiki"]


class GetNodeQuery(BaseModel):
    """节点详情GET请求参数
    https://open.feishu.cn/document/server-docs/docs/wiki-v2/space-node/get_node
    """

    token: str
    """"知识库或文档的 token。
        了解如何获取 token https://open.feishu.cn/document/server-docs/docs/faq#560bf735"""
    obj_type: NodeTypes = "wiki"
    """文档类型。不传时默认以wiki类型查询。
        可选值有：doc：旧版文档
        docx：新版文档
        sheet：表格
        mindnote：思维导图
        bitable：多维表格
        file：文件
        slides：幻灯片
        wiki：知识库节点"""



from typing import Union

import httpx

from slark.resources._resources import APIResource, AsyncAPIResource
from slark.resources.api_path import API_PATH
from slark.types.spreadsheets import (
    GetAllWorksheetsInfoResponse,
    GetWorksheetInfoResponse,
)


class AsyncWorksheet(AsyncAPIResource):
    async def get_all_worksheets_info(
        self,
        sheet_token: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetAllWorksheetsInfoResponse:
        """根据电子表格 token 获取表格中所有工作表及其属性信息，包括工作表 ID、标题、索引位置、是否被隐藏等。
        https://open.feishu.cn/document/server-docs/docs/sheets-v3/spreadsheet-sheet/query

        Args:
            sheet_token (str): 	电子表格的 token。
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            GetAllWorksheetsInfoResponse: 所有工作表信息。
        """
        return await self._get(
            API_PATH.spreadsheets.get_all_worksheets.format(spreadsheet_token=sheet_token),
            cast_to=GetAllWorksheetsInfoResponse,
            options={"timeout": timeout},
        )

    async def get_worksheet_info(
        self,
        sheet_token: str,
        *,
        sheet_id: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetWorksheetInfoResponse:
        return await self._get(
            API_PATH.spreadsheets.get_worksheet_info.format(
                spreadsheet_token=sheet_token, sheet_id=sheet_id
            ),
            options={"timeout": timeout},
            cast_to=GetWorksheetInfoResponse,
        )


class Worksheet(APIResource):
    def get_all_worksheets_info(
        self,
        sheet_token: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetAllWorksheetsInfoResponse:
        """根据电子表格 token 获取表格中所有工作表及其属性信息，包括工作表 ID、标题、索引位置、是否被隐藏等。
        https://open.feishu.cn/document/server-docs/docs/sheets-v3/spreadsheet-sheet/query

        Args:
            sheet_token (str): 	电子表格的 token。
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            GetAllWorksheetsInfoResponse: 所有工作表信息。
        """
        return self._get(
            API_PATH.spreadsheets.get_all_worksheets.format(spreadsheet_token=sheet_token),
            cast_to=GetAllWorksheetsInfoResponse,
            options={"timeout": timeout},
        )

    def get_worksheet_info(
        self,
        sheet_token: str,
        *,
        sheet_id: str,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetWorksheetInfoResponse:
        return self._get(
            API_PATH.spreadsheets.get_worksheet_info.format(
                spreadsheet_token=sheet_token, sheet_id=sheet_id
            ),
            options={"timeout": timeout},
            cast_to=GetWorksheetInfoResponse,
        )



from typing_extensions import Literal

from ..base import BaseElement


class PersonElement(BaseElement):
    tag: str = "person"
    size: Literal["extra_small", "small", "medium", "large"] = "medium"
    show_avatar: bool = False
    show_name: bool = True
    style: Literal["normal", "capsule"] = "normal"
    user_id: str


from typing import List, Union

from ..base import BaseElement
from ..icon import IconElement
from .image import ImageElement
from .plain_text import PlainTextElement


class NoteElement(BaseElement):
    tag: str = "note"
    elements: List[Union[IconElement, PlainTextElement, ImageElement]]



from typing import Any, Dict

import pydantic


class BaseModel(pydantic.BaseModel):
    model_config = pydantic.ConfigDict(
        use_enum_values=True,
        arbitrary_types_allowed=True,
        extra="allow",
    )

    def model_dump(
        self,
        *,
        exclude_none: bool = True,
        **kwargs: Any,
    ) -> Dict[str, Any]:
        return super().model_dump(exclude_none=exclude_none, **kwargs)



from typing import Union

from pydantic import BaseModel, ConfigDict
from typing_extensions import Literal


class SegmentStyle(BaseModel):
    bold: bool = False
    italic: bool = False
    strikeThrough: bool = False
    underline: bool = False
    foreColor: str = "#000000"
    fontSize: int = 10


class BaseCell(BaseModel):
    type: str
    text: str
    segment_style: Union[SegmentStyle, None] = None

    model_config = ConfigDict(extra="allow")


class LinkWithText(BaseCell):
    link: str
    type: str = "url"


class Mention(BaseCell):
    """只支持@同租户的用户;单次请求最多支持同时@50人；
    notify:是否发送飞书消息，没有阅读权限的用户不会收到飞书消息；

    grantReadPermission:是否赋予该用户阅读权限（仅在独立表格中支持该字段）；仅当拥有文档分享权限时可用；

    textType:指定text字段的传入的内容，可选email，openId，unionId；

    text:需要@的人的信息，由textType指定"""

    type: str = "mention"
    text_type: str
    notify: bool = True
    grantReadPermission: bool = True


class Formula(BaseCell):
    """text字段为对应的公式，暂不支持跨表引用公式（IMPORTRANGE）"""

    type: str = "formula"


class MentionDocument(BaseCell):
    """textType:固定为fileToken

    text:文档token

    objType:文档类型，可选sheet,doc,slide,bitable,mindnote"""

    type: str = "mention"
    textType: str = "fileToken"
    objType: Literal["sheet", "doc", "slide", "bitable", "mindnote"]


CellTypes = Union[None, str, int, float, bool, LinkWithText, Mention, Formula, MentionDocument]



from typing import List, Union

from typing_extensions import Literal

from slark.types._common import BaseModel

from .common import CellTypes


class WriteDataValueRange(BaseModel):
    range: str
    """	写入数据的范围。格式为 <sheetId>!<开始位置>:<结束位置>。其中：

    sheetId 为工作表 ID，通过获取工作表 获取。

    <开始位置>:<结束位置> 为工作表中单元格的范围，数字表示行索引，字母表示列索引。如 A2:B2 表示该工作表第 2 行的 A 列到 B 列。range支持四种写法，详情参考电子表格概述。

    注意：range 所指定的范围需要大于等于写入的数据所占用的范围。"""
    values: List[List[CellTypes]]


class PrependDataBody(BaseModel):
    """指定工作表中的范围和在该范围中插入的数据。"""

    valueRange: WriteDataValueRange


class AppendDataBody(PrependDataBody):
    pass


class AppendDataParams(BaseModel):
    insertDataOption: Literal["OVERWRITE", "INSERT_ROWS"] = "OVERWRITE"
    """指定追加数据的方式，默认值为 OVERWRITE，即若空行数量小于追加数据的行数，则会覆盖已有数据。可选值：

    OVERWRITE：若空行的数量小于追加数据的行数，则会覆盖已有数据
    INSERT_ROWS：插入足够数量的行后再进行数据追加"""


class ReadSingleRangeParams(BaseModel):
    valueRenderOption: Union[str, None] = None
    """指定单元格数据的格式。可选值如下所示。当参数缺省时，默认不进行公式计算，返回公式本身，且单元格为数值格式。

    ToString：返回纯文本的值（数值类型除外）
    Formula：单元格中含有公式时，返回公式本身
    FormattedValue：计算并格式化单元格
    UnformattedValue：计算但不对单元格进行格式化"""

    dateTimeRenderOption: Union[str, None] = None
    """指定数据类型为日期、时间、或时间日期的单元格数据的格式。

    若不传值，默认返回浮点数值，整数部分为自 1899 年 12 月 30 日以来的天数；小数部分为该时间占 24 小时的份额。例如：若时间为 1900 年 1 月 1 日中午 12 点，则默认返回 2.5。其中，2 表示 1900 年 1 月 1 日为 1899 年12 月 30 日之后的 2 天；0.5 表示 12 点占 24 小时的二分之一，即 12/24=0.5。
    可选值为 FormattedString，此时接口将计算并对日期、时间、或时间日期类型的数据格式化并返回格式化后的字符串，但不会对数字进行格式化。"""

    user_id_type: Union[str, None] = None
    """当单元格中包含@用户等涉及用户信息的元素时，该参数可指定返回的用户 ID 类型。默认为 lark_id，建议选择 open_id 或 union_id。了解更多，参考用户身份概述。可选值：
    open_id：标识一个用户在某个应用中的身份。同一个用户在不同应用中的 Open ID 不同。了解更多：如何获取 Open ID
    union_id：标识一个用户在某个应用开发商下的身份。同一用户在同一开发商下的应用中的 Union ID 是相同的，在不同开发商下的应用中的 Union ID 是不同的。通过 Union ID，应用开发商可以把同个用户在多个应用中的身份关联起来。了解更多：如何获取 Union ID"""


class ReadMultiRangeParams(BaseModel):
    ranges: str
    """多个查询范围，范围之间使用逗号分隔，如 Q7PlXT!A2:B6,0b6377!B1:C8。range 的格式为 <sheetId>!<开始位置>:<结束位置>。其中：

    sheetId 为工作表 ID，通过 [获取工作表](https://open.feishu.cn/document/server-docs/docs/sheets-v3/spreadsheet-sheet/query) 获取
    <开始位置>:<结束位置> 为工作表中单元格的范围，数字表示行索引，字母表示列索引。如 A2:B2 表示该工作表第 2 行的 A 列到 B 列。
    range支持四种写法，详情参考[电子表格概述](https://open.feishu.cn/document/server-docs/docs/sheets-v3/overview)
    注意：若使用 <sheetId>!<开始单元格>:<结束列> 的写法时，仅支持获取 100 列数据。"""
    valueRenderOption: Union[str, None] = None
    """指定单元格数据的格式。可选值如下所示。当参数缺省时，默认不进行公式计算，返回公式本身，且单元格为数值格式。

    ToString：返回纯文本的值（数值类型除外）
    Formula：单元格中含有公式时，返回公式本身
    FormattedValue：计算并格式化单元格
    UnformattedValue：计算但不对单元格进行格式化"""

    dateTimeRenderOption: Union[str, None] = None
    """指定数据类型为日期、时间、或时间日期的单元格数据的格式。

    若不传值，默认返回浮点数值，整数部分为自 1899 年 12 月 30 日以来的天数；小数部分为该时间占 24 小时的份额。例如：若时间为 1900 年 1 月 1 日中午 12 点，则默认返回 2.5。其中，2 表示 1900 年 1 月 1 日为 1899 年12 月 30 日之后的 2 天；0.5 表示 12 点占 24 小时的二分之一，即 12/24=0.5。
    可选值为 FormattedString，此时接口将计算并对日期、时间、或时间日期类型的数据格式化并返回格式化后的字符串，但不会对数字进行格式化。"""

    user_id_type: Union[str, None] = None
    """当单元格中包含@用户等涉及用户信息的元素时，该参数可指定返回的用户 ID 类型。默认为 lark_id，建议选择 open_id 或 union_id。了解更多，参考用户身份概述。可选值：
    open_id：标识一个用户在某个应用中的身份。同一个用户在不同应用中的 Open ID 不同。了解更多：如何获取 Open ID
    union_id：标识一个用户在某个应用开发商下的身份。同一用户在同一开发商下的应用中的 Union ID 是相同的，在不同开发商下的应用中的 Union ID 是不同的。通过 Union ID，应用开发商可以把同个用户在多个应用中的身份关联起来。了解更多：如何获取 Union ID"""


class WriteSingleRangeBody(BaseModel):
    valueRange: WriteDataValueRange


class WriteMultiRangeBody(BaseModel):
    valueRanges: List[WriteDataValueRange]



from typing import Union

import httpx

from slark.resources._resources import AsyncAPIResource
from slark.resources.api_path import API_PATH
from slark.types.bitables.request import UpdateBitableMetaBody
from slark.types.bitables.response import GetBitableMetaResponse, UpdateBitableMetaResponse


class AsyncMeta(AsyncAPIResource):
    async def get(
        self,
        app_token: str,
        *,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetBitableMetaResponse:
        """获取多维表格元数据 https://open.feishu.cn/document/server-docs/docs/bitable-v1/app/get

        Args:
            app_token (str): 多维表格的 app_token
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            GetBitableMetaResponse: 多维表格元数据
        """
        return await self._get(
            API_PATH.bitables.get_bitable_meta.format(app_token=app_token),
            cast_to=GetBitableMetaResponse,
            options={"timeout": timeout},
        )

    async def update(
        self,
        app_token: str,
        *,
        name: Union[str, None] = None,
        is_advanced: Union[bool, None] = None,
    ) -> UpdateBitableMetaResponse:
        """更新多维表格元数据 https://open.feishu.cn/document/server-docs/docs/bitable-v1/app/update

        Args:
            app_token (str): 多维表格的 app_token
            name (Union[str, None], optional): 更新表格的名字. Defaults to None.
            is_advanced (Union[bool, None], optional): 是否开启高级权限. Defaults to None.

        Returns:
            UpdateBitableMetaResponse: 更新返回值
        """
        return await self._put(
            API_PATH.bitables.update_bitable_meta.format(app_token=app_token),
            body=UpdateBitableMetaBody(
                name=name,
                is_advanced=is_advanced,
            ).model_dump(),
            cast_to=UpdateBitableMetaResponse,
        )


class Meta(AsyncAPIResource):
    def get(
        self,
        app_token: str,
        *,
        timeout: Union[httpx.Timeout, None] = None,
    ) -> GetBitableMetaResponse:
        """获取多维表格元数据 https://open.feishu.cn/document/server-docs/docs/bitable-v1/app/get

        Args:
            app_token (str): 多维表格的 app_token
            timeout (Union[httpx.Timeout, None], optional): Timeout. Defaults to None.

        Returns:
            GetBitableMetaResponse: 多维表格元数据
        """
        return self._get(
            API_PATH.bitables.get_bitable_meta.format(app_token=app_token),
            cast_to=GetBitableMetaResponse,
            options={"timeout": timeout},
        )

    def update(
        self,
        app_token: str,
        *,
        name: Union[str, None] = None,
        is_advanced: Union[bool, None] = None,
    ) -> UpdateBitableMetaResponse:
        """更新多维表格元数据 https://open.feishu.cn/document/server-docs/docs/bitable-v1/app/update

        Args:
            app_token (str): 多维表格的 app_token
            name (Union[str, None], optional): 更新表格的名字. Defaults to None.
            is_advanced (Union[bool, None], optional): 是否开启高级权限. Defaults to None.

        Returns:
            UpdateBitableMetaResponse: 更新返回值
        """
        return self._put(
            API_PATH.bitables.update_bitable_meta.format(app_token=app_token),
            body=UpdateBitableMetaBody(
                name=name,
                is_advanced=is_advanced,
            ).model_dump(),
            cast_to=UpdateBitableMetaResponse,
        )



import asyncio
import inspect
import threading


def run_in_thread(func):
    def wrapper(*args, **kwargs):
        if inspect.iscoroutinefunction(func):
            _thread = threading.Thread(target=asyncio.run, args=(func(*args, **kwargs),))
        else:
            _thread = threading.Thread(target=func, args=args, kwargs=kwargs)
        _thread.start()

    return wrapper


if __name__ == "__main__":
    import time

    @run_in_thread
    def test(test=1):
        time.sleep(test)
        print("Hello, world!")

    test(test=1)
    print("Main thread")
    time.sleep(2)



from typing import List, Union

from pydantic import field_validator

from slark.types._common import BaseModel
from slark.types.bitables.common import FieldPropertyType, FieldType, UIType


class TableFieldDescription(BaseModel):
    disable_sync: Union[bool, None] = None
    """是否禁止同步，如果为true，表示禁止同步该描述内容到表单的问题描述

    示例值：true

    默认值：true"""

    text: Union[str, None] = None
    """字段描述内容，支持换行\n

    示例值："请按 name_id 格式填写\n例如：“Alice_20202020”"""


class TableField(BaseModel):
    field_name: str
    """字段名,示例值："文本"""
    type: FieldType
    ui_type: Union[UIType, None] = None
    property: Union[FieldPropertyType, None] = None
    description: Union[TableFieldDescription, None] = None


class TableData(BaseModel):
    name: Union[str, None] = None
    """数据表名称

    请注意：

    名称中的首尾空格将会被去除。
    示例值："table1"

    数据校验规则：

    长度范围：1 字符 ～ 100 字符"""
    default_view_name: Union[str, None] = None
    """默认表格视图的名称，不填则默认为 表格。

    请注意：

    名称中的首尾空格将会被去除。
    名称中不允许包含 [ ] 两个字符"""

    fields: Union[List[TableField]] = None
    """数据表的初始字段。

    请注意：

    如果 default_view_name 字段和 fields 字段都不填写，将会创建一个仅包含索引列的空数据表。

    如果指定了 fields 字段，将会创建一个包含初始字段的数据表且默认第一个字段为索引列。

    数据校验规则：

    长度范围：1 ～ 300"""

    @field_validator("name")
    def validate_name(cls, v: str):
        v = v.strip()
        if not v or len(v) > 100:
            raise ValueError("name length should be 1-100")
        return v

    @field_validator("default_view_name")
    def validate_default_view_name(cls, v: str):
        v = v.strip()
        if "[" in v or "]" in v:
            raise ValueError("default_view_name should not contain [ or ]")
        return v

    @field_validator("fields")
    def validate_fields(cls, v: List[TableField]):
        if v is not None and len(v) > 300:
            raise ValueError("fields length should be 1-300")
        return v
